import sys
import os
import re
import argparse
import tempfile
import subprocess
import ssgcommon
import lxml.etree as ET
from ConfigParser import SafeConfigParser

import idtranslate_module as idtranslate

SHARED_OVAL = re.sub('shared.*', 'shared', __file__) + '/checks/oval/'
timestamp = ssgcommon.timestamp


footer = ssgcommon.oval_footer
ovalns = ssgcommon.oval_namespace

try:
    from openscap import oscap_get_version
    if oscap_get_version() < 1.2:
        oval_version = "5.10"
    else:
        oval_version = "5.11"
except ImportError:
    oval_version = "5.10"

# globals, to make recursion easier in case we encounter extend_definition
definitions = ET.Element("definitions")
tests = ET.Element("tests")
objects = ET.Element("objects")
states = ET.Element("states")
variables = ET.Element("variables")


# append new child ONLY if it's not a duplicate
def append(element, newchild):
    newid = newchild.get("id")
    existing = element.find(".//*[@id='" + newid + "']")
    if existing is not None:
        if not silent_mode:
            sys.stderr.write("Notification: this ID is used more than once " +
                             "and should represent equivalent elements: " +
                             newid + "\n")
    else:
        element.append(newchild)


def add_oval_elements(body, header):
    """Add oval elements to the global Elements defined above"""

    tree = ET.fromstring(header + body + footer)
    tree = replace_external_vars(tree)
    # parse new file(string) as an etree, so we can arrange elements
    # appropriately
    for childnode in tree.findall("./{%s}def-group/*" % ovalns):
        # print "childnode.tag is " + childnode.tag
        if childnode.tag is ET.Comment:
            continue
        if childnode.tag == ("{%s}definition" % ovalns):
            append(definitions, childnode)
            defname = childnode.get("id")
            # extend_definition is a special case:  must include a whole other
            # definition
            for defchild in childnode.findall(".//{%s}extend_definition" % ovalns):
                defid = defchild.get("definition_ref")
                extend_ref = find_testfile(defid+".xml")
                includedbody = read_ovaldefgroup_file(extend_ref)
                # recursively add the elements in the other file
                add_oval_elements(includedbody, header)
        if childnode.tag.endswith("_test"):
            append(tests, childnode)
        if childnode.tag.endswith("_object"):
            append(objects, childnode)
        if childnode.tag.endswith("_state"):
            append(states, childnode)
        if childnode.tag.endswith("_variable"):
            append(variables, childnode)
    return defname


def replace_external_vars(tree):
    """Replace external_variables with local_variables, so the definition can be
       tested independently of an XCCDF file"""

    # external_variable is a special case: we turn it into a local_variable so
    # we can test
    for node in tree.findall(".//{%s}external_variable" % ovalns):
        print ("External_variable with id : " + node.get("id"))
        extvar_id = node.get("id")
        # for envkey, envval in os.environ.iteritems():
        #     print envkey + " = " + envval
        # sys.exit()
        if extvar_id not in os.environ.keys():
            print ("External_variable specified, but no value provided via " \
                   "environment variable")
            sys.exit(2)
        # replace tag name: external -> local
        node.tag = "{%s}local_variable" % ovalns
        literal = ET.Element("literal_component")
        literal.text = os.environ[extvar_id]
        node.append(literal)
        # TODO: assignment of external_variable via environment vars, for
        # testing
    return tree


def find_testfile(testfile):
    """Find OVAL files in CWD or shared/oval"""
    for path in ['.', SHARED_OVAL]:
        for root, folder, files in os.walk(path):
            searchfile = root + '/' + testfile
            if not os.path.isfile(searchfile):
                searchfile = ""
            else:
                testfile = searchfile.strip()
                # Most likely found file, exit this loop
                break

    if not os.path.isfile(testfile):
        print ("ERROR: %s does not exist! Please specify a valid OVAL file.") % testfile
        sys.exit(1)

    return testfile


def read_ovaldefgroup_file(testfile):
    """Read oval files"""
    with open(testfile, 'r') as test_file:
        body = test_file.read()
    return body


def parse_options():
    usage = "usage: %(prog)s [options] definition_file.xml"
    parser = argparse.ArgumentParser(usage=usage, version="%(prog)s ")
    # only some options are on by default

    parser.add_argument("--oval_version", default=oval_version,
                        dest="oval_version", action="store",
                        help="OVAL version to use. Example: 5.11, 5.10, ... \
                        [Default: %(default)s]")
    parser.add_argument("-q", "--quiet", "--silent", default=False,
                        action="store_true", dest="silent_mode",
                        help="Don't show any output when testing OVAL files")
    parser.add_argument("xmlfile", metavar="XMLFILE", help="OVAL XML file")
    args = parser.parse_args()

    return args


def main():
    global definitions
    global tests
    global objects
    global states
    global variables
    global silent_mode

    args = parse_options()
    silent_mode = args.silent_mode
    oval_version = args.oval_version

    testfile = args.xmlfile
    header = ssgcommon.oval_generated_header("testoval.py", oval_version, "0.0.1")
    testfile = find_testfile(testfile)
    body = read_ovaldefgroup_file(testfile)
    defname = add_oval_elements(body, header)
    ovaltree = ET.fromstring(header + footer)

    # append each major element type, if it has subelements
    for element in [definitions, tests, objects, states, variables]:
        if element.getchildren():
            ovaltree.append(element)
    # re-map all the element ids from meaningful names to meaningless
    # numbers
    testtranslator = idtranslate.IDTranslator("scap-security-guide.testing")
    ovaltree = testtranslator.translate(ovaltree)
    (ovalfile, fname) = tempfile.mkstemp(prefix=defname, suffix=".xml")
    os.write(ovalfile, ET.tostring(ovaltree))
    os.close(ovalfile)
    if not silent_mode:
        print ("Evaluating with OVAL tempfile: " + fname)
        print ("OVAL Schema Version: %s" % oval_version)
        print ("Writing results to: " + fname + "-results")
    cmd = "oscap oval eval --results " + fname + "-results " + fname
    oscap_child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
    cmd_out = oscap_child.communicate()[0]
    if not silent_mode:
        print cmd_out
    if oscap_child.returncode != 0:
        if not silent_mode:
            print ("Error launching 'oscap' command: \n\t" + cmd)
        sys.exit(2)
    if 'false' in cmd_out:
        # at least one from the evaluated OVAL definitions evaluated to
        # 'false' result, exit with '1' to indicate OVAL scan FAIL result
        sys.exit(1)
    # perhaps delete tempfile?
    definitions = ET.Element("definitions")
    tests = ET.Element("tests")
    objects = ET.Element("objects")
    states = ET.Element("states")
    variables = ET.Element("variables")

    # 'false' keyword wasn't found in oscap's command output
    # exit with '0' to indicate OVAL scan TRUE result
    sys.exit(0)



#!/usr/bin/env python2

from __future__ import print_function

import argparse
import sys
import os


# Put shared python modules in path
sys.path.insert(0, os.path.join(
        os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
        "modules"))
import idtranslate_module as idtranslate
import ssgcommon

ET = ssgcommon.ElementTree


import parse_oval

# This script requires two arguments: an "unlinked" XCCDF file and an ID name
# scheme. This script is designed to convert and synchronize check IDs
# referenced from the XCCDF document for the supported checksystems, which are
# currently OVAL and OCIL.  The IDs are to be converted from strings to
# meaningless numbers.


oval_ns = ssgcommon.oval_namespace
oval_cs = ssgcommon.oval_namespace
xccdf_ns = ssgcommon.XCCDF11_NS
cce_uri = ssgcommon.cce_uri


OVAL_TO_XCCDF_DATATYPE_CONSTRAINTS = {
    'int': 'number',
    'float': 'number',
    'boolean': 'boolean',
    'string': 'string',
    'evr_string': 'string',
    'version': 'string',
    'ios_version': 'string',
    'fileset_revision': 'string',
    'binary': 'string'
}


class FileLinker(object):
    CHECK_SYSTEM = None
    CHECK_NAMESPACE = None

    def __init__(self, translator, xccdftree, checks):
        self.translator = translator
        self.checks_related_to_us = self._get_related_checks(checks)
        self.fname = self._get_input_fname()
        self.tree = None
        self.linked_fname = self.fname.replace("unlinked", "linked")
        self.xccdftree = xccdftree

    def _get_related_checks(self, checks):
        return [ch for ch in checks if ch.get("system") == self.CHECK_SYSTEM]

    def _get_fnames_from_related_checks(self):
        checkfiles = set()
        for check in self.checks_related_to_us:
            # Include the file in the particular check system only if it's NOT
            # a remotely located file (to allow OVAL checks to reference http://
            # and https:// formatted URLs)
            checkcontentref = ssgcommon.get_check_content_ref_if_exists_and_not_remote(check)
            if checkcontentref is not None:
                checkfiles.add(checkcontentref.get("href"))
        return checkfiles

    def _get_input_fname(self):
        fnames = self._get_fnames_from_related_checks()
        if len(fnames) > 1:
            msg = ("referencing more than one file per check system "
                   "is not yet supported by this script.")
            raise ssgcommon.SSGError(msg)
        return fnames.pop() if fnames else None

    def save_linked_tree(self):
        assert self.tree is not None, \
            "There is no tree to save, you have probably skipped the linking phase"
        ssgcommon.ElementTree.ElementTree(self.tree).write(self.linked_fname)

    def _get_checkid_string(self):
        raise NotImplementedError()

    def add_missing_check_exports(self, check, checkcontentref):
        pass

    def link_xccdf(self):
        for check in self.checks_related_to_us:
            checkcontentref = ssgcommon.get_check_content_ref_if_exists_and_not_remote(check)
            if checkcontentref is None:
                continue

            self.add_missing_check_exports(check, checkcontentref)

            checkexports = check.findall("./{%s}check-export" % xccdf_ns)

            self._link_xccdf_checkcontentref(checkcontentref, checkexports)

    def _link_xccdf_checkcontentref(self, checkcontentref, checkexports):
        checkid = self.translator.generate_id(
            self._get_checkid_string(), checkcontentref.get("name"))
        checkcontentref.set("name", checkid)
        checkcontentref.set("href", self.linked_fname)

        variable_str = "{%s}variable" % self.CHECK_NAMESPACE
        for checkexport in checkexports:
            newexportname = self.translator.generate_id(
                variable_str, checkexport.get("export-name"))
            checkexport.set("export-name", newexportname)


class OVALFileLinker(FileLinker):
    CHECK_SYSTEM = oval_cs
    CHECK_NAMESPACE = oval_ns

    def __init__(self, translator, xccdftree, checks):
        super(OVALFileLinker, self).__init__(translator, xccdftree, checks)
        self.oval_groups = None

    def _get_checkid_string(self):
        return "{%s}definition" % self.CHECK_NAMESPACE

    def link(self):
        self.oval_groups = parse_oval.get_container_oval_groups(self.fname)
        self.tree = ssgcommon.parse_xml_file(self.fname)
        try:
            self._link_oval_tree()

            # Verify if CCE identifiers present in the XCCDF follow the required form
            # (either CCE-XXXX-X, or CCE-XXXXX-X). Drop from XCCDF those who don't follow it
            verify_correct_form_of_referenced_cce_identifiers(self.xccdftree)
        except ssgcommon.SSGError as exc:
            raise ssgcommon.SSGError(
                "Error processing {0}: {1}"
                .format(self.fname, str(exc)))
        self.tree = self.translator.translate(self.tree, store_defname=True)

    def _link_oval_tree(self):
        xccdf_to_cce_id_mapping = create_xccdf_id_to_cce_id_mapping(self.xccdftree)

        indexed_oval_defs = ssgcommon.map_elements_to_their_ids(
            self.tree, ".//{0}".format(self._get_checkid_string()))

        drop_oval_checks_extending_non_existing_checks(
            self.tree, self.oval_groups, indexed_oval_defs)

        self._add_cce_id_refs_to_oval_checks(xccdf_to_cce_id_mapping)

        # Verify all by XCCDF referenced (local) OVAL checks are defined in OVAL file
        # If not drop the <check-content> OVAL checksystem reference from XCCDF
        self._ensure_by_xccdf_referenced_oval_def_is_defined_in_oval_file(
            indexed_oval_defs)

        check_and_correct_xccdf_to_oval_data_export_matching_constraints(self.xccdftree, self.tree)

    def _add_cce_id_refs_to_oval_checks(self, idmappingdict):
        """
        For each XCCDF rule ID having <ident> CCE set and
        having OVAL check implemented (remote OVAL isn't sufficient!)
        add a new <reference> element into the OVAL definition having the
        following form:

        <reference source="CCE" ref_id="CCE-ID" />

        where "CCE-ID" is the CCE identifier for that particular rule
        retrieved from the XCCDF file
        """
        ovalrules = self.tree.findall(".//{0}".format(self._get_checkid_string()))
        for rule in ovalrules:
            ovalid = rule.get("id")
            assert ovalid is not None, \
                "An OVAL rule doesn't have an ID"

            ovaldesc = rule.find(".//{%s}description" % self.CHECK_NAMESPACE)
            assert ovaldesc is not None, \
                "OVAL rule '{0}' doesn't have a description, which is mandatory".format(ovalid)

            if ovalid not in idmappingdict:
                print("OVAL ID '{0}' has not XCCDF rule ID <ident> CCE set"
                      .format(ovalid), file=sys.stderr)
                continue

            xccdfcceid = idmappingdict[ovalid]
            if not ssgcommon.cce_is_valid(xccdfcceid):
                msg = "The CCE ID {0} is not in valid form.".format(xccdfcceid)
                raise ssgcommon.SSGError(msg)

            # Then append the <reference source="CCE" ref_id="CCE-ID" /> element right
            # after <description> element of specific OVAL check
            ccerefelem = ssgcommon.ElementTree.Element(
                'reference', ref_id=xccdfcceid, source="CCE")
            ovaldesc.addnext(ccerefelem)
            # Sanity check if appending succeeded
            if ccerefelem.getprevious() is not ovaldesc:
                msg = "Failed to add CCE ID to {0}.".format(ovalid)
                raise ssgcommon.SSGError(msg)

    def get_nested_definitions(self, oval_def_id):
        processed_def_ids = set()
        queue = set([oval_def_id])
        while queue:
            def_id = queue.pop()
            processed_def_ids.add(def_id)
            definition_tree = self.oval_groups["definitions"].get(def_id)
            extensions = parse_oval.find_extending_defs(self.oval_groups, definition_tree)
            if not extensions:
                continue
            queue |= extensions - processed_def_ids
        return processed_def_ids

    def add_missing_check_exports(self, check, checkcontentref):
        check_name = checkcontentref.get("name")
        if check_name is None:
            return
        oval_def = self.oval_groups["definitions"].get(check_name)
        if oval_def is None:
            return
        all_vars = set()
        for def_id in self.get_nested_definitions(check_name):
            extended_def = self.oval_groups["definitions"].get(def_id)
            all_vars |= parse_oval.resolve_definition(self.oval_groups, extended_def)
        for varname in all_vars:
            export = ET.Element("{%s}check-export" % xccdf_ns)
            export.attrib["export-name"] = varname
            export.attrib["value-id"] = varname
            check.insert(0, export)

    def _ensure_by_xccdf_referenced_oval_def_is_defined_in_oval_file(
            self, indexed_oval_defs):
        # Ensure all OVAL checks referenced by XCCDF are implemented in OVAL file
        # Drop the reference from XCCDF to OVAL definition if:
        # * Particular OVAL definition isn't present in OVAL file,
        # * That OVAL definition doesn't constitute a remote OVAL
        #   (@href of <check-content-ref> doesn't start with 'http'

        for xccdfid, rule in rules_with_ids_generator(self.xccdftree):
            # Search OVAL ID in OVAL document
            ovalid = indexed_oval_defs.get(xccdfid)
            if ovalid is not None:
                # The OVAL check was found, we can continue
                continue

            for check in rule.findall(".//{%s}check" % (xccdf_ns)):
                if check.get("system") != oval_cs:
                    continue

                if ssgcommon.get_check_content_ref_if_exists_and_not_remote(check) is None:
                    continue

                # For local OVAL drop the reference to OVAL definition from XCCDF document
                # in the case:
                # * OVAL definition is referenced from XCCDF file,
                # * But not defined in OVAL file
                print("WARNING: OVAL check '{0}' was not found, removing "
                      "<check-content> element from the XCCDF rule."
                      .format(xccdfid), file=sys.stderr)
                rule.remove(check)


class OCILFileLinker(FileLinker):
    CHECK_SYSTEM = ssgcommon.ocil_cs
    CHECK_NAMESPACE = ssgcommon.ocil_namespace

    def _get_checkid_string(self):
        return "{%s}questionnaire" % self.CHECK_NAMESPACE

    def link(self):
        self.tree = ssgcommon.parse_xml_file(self.fname)
        self.tree = self.translator.translate(self.tree, store_defname=True)


def _find_identcce(rule):
    for ident in rule.findall("./{%s}ident" % xccdf_ns):
        if ident.get("system") == cce_uri:
            return ident
    return None


def rules_with_ids_generator(xccdftree):
    xccdfrules = xccdftree.findall(".//{%s}Rule" % xccdf_ns)
    for rule in xccdfrules:
        xccdfid = rule.get("id")
        if xccdfid is None:
            continue
        yield xccdfid, rule


def create_xccdf_id_to_cce_id_mapping(xccdftree):
    #
    # Create dictionary having form of
    #
    # 'XCCDF ID' : 'CCE ID'
    #
    # for each XCCDF rule having <ident system='http://cce.mitre.org'>CCE-ID</ident>
    # element set in the XCCDF document
    xccdftocce_idmapping = {}

    for xccdfid, rule in rules_with_ids_generator(xccdftree):
        identcce = _find_identcce(rule)
        if identcce is None:
            continue

        xccdftocce_idmapping[xccdfid] = identcce.text

    return xccdftocce_idmapping


def get_nonexisting_check_definition_extends(definition, indexed_oval_defs):
    # TODO: handle multiple levels of referrals.
    # OVAL checks that go beyond one level of extend_definition won't be properly identified
    for extdefinition in definition.findall(".//{%s}extend_definition" % oval_ns):
        # Verify each extend_definition in the definition
        extdefinitionref = extdefinition.get("definition_ref")

        # Search the OVAL tree for a definition with the referred ID
        referreddefinition = indexed_oval_defs.get(extdefinitionref)

        if referreddefinition is None:
            # There is no oval satisfying the extend_definition referal
            return extdefinitionref
    return None


def drop_oval_checks_extending_non_existing_checks(ovaltree, oval_groups, indexed_oval_defs):
    # Incomplete OVAL checks are as useful as non existing checks
    # Here we check if all extend_definition refs from a definition exists in local OVAL file
    definitions = ovaltree.find(".//{%s}definitions" % oval_ns)
    defstoremove = set()
    for definition in definitions:
        nonexisting_ref = get_nonexisting_check_definition_extends(definition, indexed_oval_defs)
        if nonexisting_ref is not None:
            print("WARNING: OVAL definition '{0}' extends non-existing '{1}', "
                  "removing it from OVAL definitions."
                  .format(definition.get("id"), nonexisting_ref),
                  file=sys.stderr)
            defstoremove.add(definition)

    for definition in defstoremove:
        del oval_groups["definitions"][definition.get("id")]
        del indexed_oval_defs[definition.get("id")]
        definitions.remove(definition)


def check_and_correct_xccdf_to_oval_data_export_matching_constraints(xccdftree, ovaltree):
    """
    Verify if <xccdf:Value> 'type' to corresponding OVAL variable
    'datatype' export matching constraint:

    http://csrc.nist.gov/publications/nistpubs/800-126-rev2/SP800-126r2.pdf#page=30&zoom=auto,69,313

    is met. Also correct the 'type' attribute of those <xccdf:Value> elements where necessary
    in order the produced content to meet this constraint.

    To correct the constraint we use simpler approach - prefer to fix
    'type' attribute of <xccdf:Value> rather than 'datatype' attribute
    of the corresponding OVAL variable since there might be additional
    OVAL variables, derived from the affected OVAL variable, and in that
    case we would need to fix the 'datatype' attribute in each of them.

    Define the <xccdf:Value> 'type' to OVAL variable 'datatype' export matching
    constraints mapping as specified in Table 16 of XCCDF v1.2 standard:

    http://csrc.nist.gov/publications/nistpubs/800-126-rev2/SP800-126r2.pdf#page=30&zoom=auto,69,313
    """
    indexed_xccdf_values = ssgcommon.map_elements_to_their_ids(
        xccdftree, ".//{%s}Value" % (xccdf_ns))

    # Loop through all <external_variables> in the OVAL document
    ovalextvars = ovaltree.findall(".//{%s}external_variable" % oval_ns)
    if ovalextvars is None:
        return

    for ovalextvar in ovalextvars:
        # Verify the found external variable has both 'id' and 'datatype' set
        if 'id' not in ovalextvar.attrib or 'datatype' not in ovalextvar.attrib:
            msg = "Invalid OVAL <external_variable> found - either without 'id' or 'datatype'."
            raise ssgcommon.SSGError(msg)

        ovalvarid = ovalextvar.get('id')
        ovalvartype = ovalextvar.get('datatype')

        # Locate the corresponding <xccdf:Value> with the same ID in the XCCDF
        xccdfvar = indexed_xccdf_values.get(ovalvarid)

        if xccdfvar is None:
            return

        xccdfvartype = xccdfvar.get('type')
        # Verify the found value has 'type' attribute set
        if xccdfvartype is None:
            msg = (
                "Invalid XCCDF variable '{0}': Missing the 'type' attribute."
                .format(xccdfvar.attrib("id")))
            raise ssgcommon.SSGError(msg)

        # This is the required XCCDF 'type' for <xccdf:Value> derived
        # from OVAL variable 'datatype' and mapping above
        reqxccdftype = OVAL_TO_XCCDF_DATATYPE_CONSTRAINTS[ovalvartype]
        # Compare the actual value of 'type' of <xccdf:Value> with the requirement
        if xccdfvartype != reqxccdftype:
            # If discrepancy is found, issue a warning
            sys.stderr.write(
                "Warning: XCCDF 'type' of \"%s\" value does "
                "not meet the XCCDF value 'type' to OVAL "
                "variable 'datatype' export matching "
                "constraint! Got: \"%s\", Expected: \"%s\". "
                "Resetting it! Set 'type' of \"%s\" "
                "<xccdf:value> to '%s' directly in the XCCDF "
                "content to dismiss this warning!" %
                (ovalvarid, xccdfvartype, reqxccdftype,
                 ovalvarid, reqxccdftype)
            )
            # And reset the 'type' attribute of such a <xccdf:Value> to the required type
            xccdfvar.attrib['type'] = reqxccdftype


def verify_correct_form_of_referenced_cce_identifiers(xccdftree):
    """
    In SSG benchmarks, the CCEs till unassigned have the form of e.g. "RHEL7-CCE-TBD"
    (or any other format possibly not matching the above two requirements)

    If this is the case for specific SSG product, drop such CCE identifiers from the XCCDF
    since they are in invalid format!
    """
    xccdfrules = xccdftree.findall(".//{%s}Rule" % xccdf_ns)
    for rule in xccdfrules:
        identcce = _find_identcce(rule)
        if identcce is None:
            continue

        cceid = identcce.text
        if not ssgcommon.cce_is_valid(cceid):
            msg = "The CCE ID {0} is not in valid form.".format(cceid)
            raise ssgcommon.SSGError(msg)


def create_parser():
    parser = argparse.ArgumentParser(
        description="This script finds check-content files (currently, "
        "OVAL and OCIL) referenced from XCCDF and synchronizes all IDs.")
    parser.add_argument("xccdf_file")
    parser.add_argument("id_name", help="ID naming scheme")
    return parser


def assert_that_check_ids_match_rule_id(checks, xccdf_rule):
    for check in checks:
        check_name = check.get("name")
        # Verify match of XCCDF vs OVAL / OCIL IDs for
        # * the case of OVAL <check>
        # * the case of OCIL <check>
        if (xccdf_rule != check_name
                and check_name is not None
                and xccdf_rule + '_ocil' != check_name
                and xccdf_rule != 'sample_rule'):
            msg_lines = ["The OVAL / OCIL ID does not match the XCCDF Rule ID!"]
            if '_ocil' in check_name:
                id_name = "OCIL ID"
            else:
                id_name = "OVAL ID"
            msg_lines.append(" {0:>14}: {1}".format(id_name, check_name))
            msg_lines.append(" {0:>14}: {1}".format("XCCDF Rule ID", xccdf_rule))
            raise ssgcommon.SSGError("\n".join(msg_lines))


def check_that_oval_and_rule_id_match(xccdftree):
    for xccdfid, rule in rules_with_ids_generator(xccdftree):
        checks = rule.find("./{%s}check" % xccdf_ns)
        if checks is None:
            print("Rule {0} doesn't have checks."
                  .format(xccdfid), file=sys.stderr)
            continue

        assert_that_check_ids_match_rule_id(checks, xccdfid)


def main():
    parser = create_parser()
    args = parser.parse_args()
    xccdffile = args.xccdf_file
    idname = args.id_name

    # Step over xccdf file, and find referenced check files
    xccdftree = ssgcommon.parse_xml_file(xccdffile)

    if 'unlinked-ocilref' not in xccdffile:
        check_that_oval_and_rule_id_match(xccdftree)

    checks = xccdftree.findall(".//{%s}check" % xccdf_ns)

    translator = idtranslate.IDTranslator(idname)

    oval_linker = OVALFileLinker(translator, xccdftree, checks)
    oval_linker.link()
    oval_linker.save_linked_tree()
    oval_linker.link_xccdf()

    ocil_linker = OCILFileLinker(translator, xccdftree, checks)
    ocil_linker.link()
    ocil_linker.save_linked_tree()
    ocil_linker.link_xccdf()

    newxccdffile = xccdffile.replace("unlinked", "linked")
    ssgcommon.ElementTree.ElementTree(xccdftree).write(newxccdffile)
    sys.exit(0)


if __name__ == "__main__":
    try:
        main()
    except ssgcommon.SSGError as exc:
        raise



#!/usr/bin/env python2
from __future__ import print_function

import atexit
import logging
import sys

import ssg_test_suite.oscap
import ssg_test_suite.virt
from ssg_test_suite.rule import get_viable_profiles
from ssg_test_suite.virt import SnapshotStack

logging.getLogger(__name__).addHandler(logging.NullHandler())


def perform_profile_check(options):
    """Perform profile check.

    Iterate over profiles in datastream and perform scanning of unaltered VM
    using every profile according to input. Also perform remediation run.

    Return value not defined, textual output and generated reports is the
    result.
    """
    dom = ssg_test_suite.virt.connect_domain(options.hypervisor,
                                             options.domain_name)
    if dom is None:
        sys.exit(1)
    snapshot_stack = SnapshotStack(dom)
    atexit.register(snapshot_stack.clear)

    snapshot_stack.create('origin')
    ssg_test_suite.virt.start_domain(dom)
    domain_ip = ssg_test_suite.virt.determine_ip(dom)

    has_worked = False
    profiles = get_viable_profiles(options.target,
                                   options.datastream,
                                   options.benchmark_id)
    if len(profiles) > 1:
        snapshot_stack.create('profile')
    for profile in profiles:
        logging.info("Evaluation of profile {0}.".format(profile))
        has_worked = True
        runner = options.remediate_using
        ssg_test_suite.oscap.run_profile(domain_ip,
                                         profile,
                                         'initial',
                                         options.datastream,
                                         options.benchmark_id,
                                         runner=runner)
        ssg_test_suite.oscap.run_profile(domain_ip,
                                         profile,
                                         'remediation',
                                         options.datastream,
                                         options.benchmark_id,
                                         runner=runner)
        ssg_test_suite.oscap.run_profile(domain_ip,
                                         profile,
                                         'final',
                                         options.datastream,
                                         options.benchmark_id,
                                         runner=runner)
        snapshot_stack.revert(delete=False)
    if not has_worked:
        logging.error("Nothing has been tested!")
    snapshot_stack.delete()
    # depending on number of profiles we have either "origin" snapshot
    # still to be reverted (multiple profiles) or we are reverted
    # completely (only one profile was run)



#
# create_mounts.py
#        generate template-based checks for partitions and partition
#        mount rights

from __future__ import print_function

import re
import sys

from template_common import FilesGenerator, UnknownTargetError


OUTPUTS_FORMAT_STRINGS = dict(
    bash="./bash/mount_option_{0}.sh",
    ansible="./ansible/mount_option_{0}.yml",
    anaconda="./anaconda/mount_option_{0}.anaconda",
    oval="./oval/mount_option_{0}.xml",
)


class Skipped(Exception):
    pass


class MountOptionTarget(object):
    TEMPLATE_FILE_BASE = None
    OUTPUT_FORMAT_STRING = None

    def __init__(self, generator, target):
        self.OUTPUT_FORMAT_STRING = OUTPUTS_FORMAT_STRINGS[target]
        self.generator = generator
        self.TEMPLATE_FILE_BASE = self.TEMPLATE_FILE_BASE.format(target=target.upper())
        self.template_file = self.TEMPLATE_FILE_BASE

        self._mount_point = ""
        self._mount_option = ""
        self._point_id = ""
        self._output_id_template = "{point_id}_{mount_option}"
        self._assert_mount_exists = True
        self._output_fname = None

    def process(self, mount_point, mount_option, assert_mount_exists):
        point_id = re.sub('[-\./]', '_', mount_point).lstrip("_")

        try:
            self._translate_input_values(mount_point, mount_option, point_id)
        except Skipped as exc:
            # This would spam the user too many times during build.
            # print("Note: {0} - {1}".format(mount_point, str(exc)), file=sys.stderr)
            return

        self._output_fname = self.OUTPUT_FORMAT_STRING.format(
            self._output_id_template.format(
                point_id=self._point_id, mount_option=self._mount_option)
        )
        self._assert_mount_exists = assert_mount_exists
        self._process()

    def _process(self):
        raise NotImplementedError("You are supposed to use a derived class.")

    def _translate_input_values(self, mount_point, mount_option, point_id):
        self._mount_point = mount_point
        self._point_id = point_id
        self._mount_option = mount_option


class RemediationTarget(MountOptionTarget):
    TEMPLATE_FILE_BASE = "./template_{target}_mount_option"

    def _process(self):
        mount_has_to_exist = "yes" if self._assert_mount_exists else "no"
        self.generator.file_from_template(
            self.template_file,
            {
                "%MOUNT_HAS_TO_EXIST%": mount_has_to_exist,
                "%MOUNTPOINT%": self._mount_point,
                "%MOUNTOPTION%": re.sub(' ', ',', self._mount_option),
            },
            self._output_fname,
            ""
        )

    def _translate_input_values(self, mount_point, mount_option, point_id):
        super(RemediationTarget, self)._translate_input_values(mount_point, mount_option, point_id)
        if mount_point.startswith("var_"):
            self._point_id = re.sub(r"^var_(.*)", r"\1s", mount_point)
            self.template_file = "{0}_var".format(self.TEMPLATE_FILE_BASE)
            self._output_id_template = "{mount_option}_{point_id}"
        elif not mount_point.startswith("/"):  # no path, but not a variable either
            raise Skipped(
                "Remediations are available only for for literal mount points, "
                "or by mount points defined by variables, thus beginning with 'var_'."
            )


class OvalTarget(MountOptionTarget):
    TEMPLATE_FILE_BASE = "./template_OVAL_mount_option"

    def __init__(self, generator):
        super(OvalTarget, self).__init__(
            generator, "oval")

    def _translate_input_values(self, mount_point, mount_option, point_id):
        super(OvalTarget, self)._translate_input_values(mount_point, mount_option, point_id)
        if mount_point.startswith("var_"):
            self._point_id = re.sub(r"^var_(.*)", r"\1s", mount_point)
            self.template_file = "{0}_{1}".format(self.TEMPLATE_FILE_BASE, self._point_id)
            self._output_id_template = "{mount_option}_{point_id}"
        elif not mount_point.startswith("/"):  # no path, but not a variable either
            point_id = re.sub(r"^(.*)", r"\1s", mount_point)
            self.template_file = "{0}_{1}".format(self.TEMPLATE_FILE_BASE, self._point_id)

    def _process(self):
        self.generator.file_from_template(
            self.template_file,
            {
                "%MOUNTPOINT%":  self._mount_point,
                "%MOUNTOPTION%": self._mount_option,
                "%POINTID%":     self._point_id,
            },
            self._output_fname,
            ""
        )


class MountOptionsGenerator(FilesGenerator):
    def __init__(self):
        self.targets = {}
        self.targets["bash"] = RemediationTarget(
            self, "bash")
        self.targets["ansible"] = RemediationTarget(
            self, "ansible")
        self.targets["anaconda"] = RemediationTarget(
            self, "anaconda")
        self.targets["oval"] = OvalTarget(self)
        super(MountOptionsGenerator, self).__init__()

    def generate(self, target, path_info):
        mount_point, mount_option = path_info[:2]
        mount_has_to_exist = True
        if len(path_info) > 2:
            assert len(path_info) == 3
            assert path_info[-1] == "create_fstab_entry_if_needed"
            mount_has_to_exist = False

        assert mount_point

        processing_entity = self.targets.get(target)
        if processing_entity is None:
            raise UnknownTargetError(target)

        processing_entity.process(mount_point, mount_option, mount_has_to_exist)

    def csv_format(self):
        return("CSV should contains lines of the format: "
               "<mount_point>,<mount_option>[,create_fstab_entry_if_needed])")



import os.path

_DIR = os.path.dirname(__file__)
_SSG_PREFIX = 'xccdf_org.ssgproject.content_'


def iterate_over_rules():
    """Iterate over directories beginning with "rule_".

    Returns:
    dir_name -- absolute path to the rule directory
    rule -- full rule id as it is present in datastream
    files -- list of files in the directory
    """
    for dir_name, directories, files in os.walk(_DIR):
        leaf_dir = os.path.basename(dir_name)
        if leaf_dir.startswith('rule_'):
            yield dir_name, _SSG_PREFIX + leaf_dir, files



#!/usr/bin/env python2
from __future__ import print_function

import atexit
import logging
import os
import os.path
import re
import shlex
import string
import subprocess
import sys

import ssg_test_suite.oscap as oscap
import ssg_test_suite.virt
from ssg_test_suite import xml_operations
from ssg_test_suite.virt import SnapshotStack
from ssg_test_suite.log import LogHelper
from data import iterate_over_rules

logging.getLogger(__name__).addHandler(logging.NullHandler())


def _parse_parameters(script):
    """Parse parameters from script header"""
    params = {'profiles': [],
              'templates': [],
              'remediation': ['all']}
    with open(script, 'r') as script_file:
        script_content = script_file.read()
        for parameter in params:
            found = re.search('^# {0} = ([ ,_\.\-\w]*)$'.format(parameter),
                              script_content,
                              re.MULTILINE)
            if found is None:
                continue
            params[parameter] = string.split(found.group(1), ', ')
    return params


def get_viable_profiles(selected_profiles, datastream, benchmark):
    """Read datastream, and return set intersection of profiles of given
    benchmark and those provided in `selected_profiles` parameter.
    """

    valid_profiles = []
    all_profiles = xml_operations.get_all_profiles_in_benchmark(
        datastream, benchmark, logging)
    for ds_profile_element in all_profiles:
        ds_profile = ds_profile_element.attrib['id']
        if 'ALL' in selected_profiles:
            valid_profiles += [ds_profile]
            continue
        for sel_profile in selected_profiles:
            if ds_profile.endswith(sel_profile):
                valid_profiles += [ds_profile]
    if not valid_profiles:
        logging.error('No profile matched with "{0}"'
                      .format(", ".join(selected_profiles)))
    return valid_profiles


def _send_scripts(rule_dir, domain_ip, *scripts_list):
    """Upload scripts to VM."""
    # scripts_list is list of absolute paths
    remote_dir = './'
    machine = "root@{0}".format(domain_ip)
    logging.debug("Uploading scripts {0}".format(scripts_list))
    rule_name = os.path.basename(rule_dir)
    log_file_name = os.path.join(LogHelper.LOG_DIR, rule_name + ".upload.log")

    command = "scp {0} {1}:{2}".format(' '.join(scripts_list),
                                       machine,
                                       remote_dir)
    with open(log_file_name, 'a') as log_file:
        subprocess.check_call(shlex.split(command),
                              stdout=log_file,
                              stderr=subprocess.STDOUT)


def _apply_script(rule_dir, domain_ip, script):
    """Run particular test script on VM and log it's output."""
    script_remote_path = os.path.join('./', script)
    machine = "root@{0}".format(domain_ip)
    logging.debug("Applying script {0}".format(script))
    rule_name = os.path.basename(rule_dir)
    log_file_name = os.path.join(LogHelper.LOG_DIR,
                                 rule_name + ".prescripts.log")

    with open(log_file_name, 'a') as log_file:
        log_file.write('##### {0} / {1} #####\n'.format(rule_name, script))

    command = "ssh {0} bash -x {1}".format(machine, script_remote_path)
    with open(log_file_name, 'a') as log_file:
        try:
            subprocess.check_call(shlex.split(command),
                                  stdout=log_file,
                                  stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as e:
            logging.error(("Rule testing script {0} "
                           "failed with exit code {1}").format(script,
                                                               e.returncode))
            return False
    return True


def _get_script_context(script):
    """Return context of the script."""
    result = re.search('.*\.([^.]*)\.[^.]*$', script)
    if result is None:
        return None
    return result.group(1)


def perform_rule_check(options):
    """Perform rule check.

    Iterate over rule-testing scenarios and utilize `oscap-ssh` to test every
    scenario. Expected result is encoded in scenario file name. In case of
    `fail` or `error` results expected, continue with remediation and
    reevaluation. Revert system to clean state using snapshots.

    Return value not defined, textual output and generated reports is the
    result.
    """
    dom = ssg_test_suite.virt.connect_domain(options.hypervisor,
                                             options.domain_name)
    if dom is None:
        sys.exit(1)
    snapshot_stack = SnapshotStack(dom)
    atexit.register(snapshot_stack.clear)

    snapshot_stack.create('origin')
    ssg_test_suite.virt.start_domain(dom)
    domain_ip = ssg_test_suite.virt.determine_ip(dom)
    scanned_something = False
    for rule_dir, rule, scripts in iterate_over_rules():
        if 'ALL' in options.target:
            # we want to have them all
            pass
        else:
            perform = False
            for target in options.target:
                if target in rule_dir:
                    perform = True
                    break
            if not perform:
                continue
        logging.info(rule)
        scanned_something = True
        logging.debug("Testing rule directory {0}".format(rule_dir))
        # get list of helper scripts (non-standard name)
        # and scenario scripts
        helpers = []
        scenarios = []
        for script in scripts:
            script_context = _get_script_context(script)
            if script_context is None:
                logging.debug('Registering helper script {0}'.format(script))
                helpers += [script]
            else:
                scenarios += [script]

        for script in scenarios:
            script_context = _get_script_context(script)
            logging.debug(('Using test script {0} '
                           'with context {1}').format(script, script_context))
            snapshot_stack.create('script')
            # copy all helper scripts, so scenario script can use them
            script_path = os.path.join(rule_dir, script)
            helper_paths = map(lambda x: os.path.join(rule_dir, x), helpers)
            _send_scripts(rule_dir, domain_ip, script_path, *helper_paths)

            if not _apply_script(rule_dir, domain_ip, script):
                logging.error("Environment failed to prepare, skipping test")
                snapshot_stack.revert()
                continue
            script_params = _parse_parameters(script_path)
            has_worked = False
            profiles = get_viable_profiles(script_params['profiles'],
                                           options.datastream,
                                           options.benchmark_id)
            if len(profiles) > 1:
                snapshot_stack.create('profile')
            for profile in profiles:
                LogHelper.preload_log(logging.INFO,
                                      ("Script {0} "
                                       "using profile {1} "
                                       "OK").format(script,
                                                    profile),
                                      log_target='pass')
                LogHelper.preload_log(logging.ERROR,
                                      ("Script {0} "
                                       "using profile {1} "
                                       "found issue:").format(script,
                                                              profile),
                                      log_target='fail')
                has_worked = True
                run_rule_checks(
                    domain_ip, profile, options.datastream,
                    options.benchmark_id, rule, script_context,
                    script, script_params, options.remediate_using,
                    options.dont_clean,
                )
                snapshot_stack.revert(delete=False)
            if not has_worked:
                logging.error("Nothing has been tested!")
            snapshot_stack.delete()
            if len(profiles) > 1:
                snapshot_stack.revert()
    if not scanned_something:
        logging.error("Rule {0} has not been found".format(options.target))


def run_rule_checks(
        domain_ip, profile, datastream, benchmark_id, rule,
        script_context, script_name, script_params, runner, dont_clean):
    def oscap_run_rule(stage, context):
        return oscap.run_rule(
            domain_ip=domain_ip,
            profile=profile,
            stage=stage,
            datastream=datastream,
            benchmark_id=benchmark_id,
            rule_id=rule,
            context=context,
            script_name=script_name,
            runner=runner,
            dont_clean=dont_clean)

    success = oscap_run_rule('initial', script_context)
    if not success:
        msg = ("The initial scan failed for rule '{}'."
               .format(rule))
        logging.error(msg)
        return False

    is_supported = set(['all'])
    is_supported.add(
        oscap.REMEDIATION_RUNNER_TO_REMEDIATION_MEANS[runner])
    supported_and_available_remediations = set(
        script_params['remediation']).intersection(is_supported)

    if (script_context not in ['fail', 'error'] or
            len(supported_and_available_remediations) == 0):
        return success

    success = oscap_run_rule('remediation', 'fixed')
    if not success:
        msg = ("The remediation failed for rule '{}'."
               .format(rule))
        logging.error(msg)
        return success

    success = oscap_run_rule('final', 'pass')
    if not success:
        msg = ("The check after remediation failed for rule '{}'."
               .format(rule))
        logging.error(msg)
    return success



#!/usr/bin/env python2

from __future__ import print_function

import json
import argparse
import os
import os.path
import sys

try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    import cElementTree as ElementTree

# Put shared python modules in path
sys.path.insert(0, os.path.join(
        os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
        "modules"))
import ssgcommon


script_desc = \
    "Obtains and displays XCCDF profile statistics. Namely number " + \
    "of rules in the profile, how many of these rules have their OVAL " + \
    "check implemented, how many have a remediation available, ..."


xccdf_ns = ssgcommon.XCCDF11_NS
oval_ns = ssgcommon.oval_namespace
bash_rem_system = ssgcommon.bash_system
ansible_rem_system = ssgcommon.ansible_system
puppet_rem_system = ssgcommon.puppet_system
anaconda_rem_system = ssgcommon.anaconda_system
cce_system = ssgcommon.cce_system
ssg_version_uri = ssgcommon.ssg_version_uri
stig_ns = ssgcommon.stig_ns
console_width = 80


class RuleStats(object):
    def __init__(self, rid=None, roval=None,
                 rbash_fix=None, ransible_fix=None,
                 rpuppet_fix=None, ranaconda_fix=None,
                 rcce=None, stig_id=None):
        self.dict = {
            'id': rid,
            'oval': roval,
            'bash_fix': rbash_fix,
            'ansible_fix': ransible_fix,
            'puppet_fix': rpuppet_fix,
            'anaconda_fix': ranaconda_fix,
            'cce': rcce,
            'stig_id': stig_id,
        }


class XCCDFBenchmark(object):
    def __init__(self, filepath):
        self.tree = None
        try:
            with open(filepath, 'r') as xccdf_file:
                file_string = xccdf_file.read()
                tree = ElementTree.fromstring(file_string)
                self.tree = tree
        except IOError as ioerr:
            print("%s" % ioerr)
            sys.exit(1)

        self.indexed_rules = {}
        for rule in self.tree.findall(".//{%s}Rule" % (xccdf_ns)):
            rule_id = rule.get("id")
            if rule_id is None:
                raise RuntimeError("Can't index a rule with no id attribute!")

            assert(rule_id not in self.indexed_rules)
            self.indexed_rules[rule_id] = rule

    def get_profile_stats(self, profile):
        """Obtain statistics for the profile"""

        # Holds the intermediary statistics for profile
        profile_stats = {
            'profile_id': None,
            'ssg_version': 0,
            'rules_count': 0,
            'implemented_ovals': [],
            'implemented_ovals_pct': 0,
            'missing_ovals': [],
            'implemented_bash_fixes': [],
            'implemented_bash_fixes_pct': 0,
            'implemented_ansible_fixes': [],
            'implemented_ansible_fixes_pct': 0,
            'implemented_puppet_fixes': [],
            'implemented_puppet_fixes_pct': 0,
            'implemented_anaconda_fixes': [],
            'implemented_anaconda_fixes_pct': 0,
            'missing_bash_fixes': [],
            'missing_ansible_fixes': [],
            'missing_puppet_fixes': [],
            'missing_anaconda_fixes': [],
            'assigned_cces': [],
            'assigned_cces_pct': 0,
            'missing_cces': [],
            'missing_stig_ids': [],
        }

        rule_stats = []
        ssg_version_elem = self.tree.find("./{%s}version[@update=\"%s\"]" %
                                          (xccdf_ns, ssg_version_uri))

        rules = []

        if profile == "all":
            # "all" is a virtual profile that selects all rules
            rules = self.indexed_rules.values()
        else:
            xccdf_profile = self.tree.find("./{%s}Profile[@id=\"%s\"]" %
                                           (xccdf_ns, profile))
            if xccdf_profile is None:
                print("No such profile \"%s\" found in the benchmark!"
                      % profile)
                print("* Available profiles:")
                profiles_avail = self.tree.findall("./{%s}Profile" % (xccdf_ns))
                for profile in profiles_avail:
                    print("** %s" % profile.get('id'))
                sys.exit(1)

            # This will only work with SSG where the (default) profile has zero
            # selected rule. If you want to reuse this for custom content, you
            # need to change this to look into Rule/@selected
            selects = xccdf_profile.findall("./{%s}select[@selected=\"true\"]" %
                                            xccdf_ns)

            for select in selects:
                rule_id = select.get('idref')
                xccdf_rule = self.indexed_rules.get(rule_id)
                if xccdf_rule is not None:
                    # it could also be a Group
                    rules.append(xccdf_rule)

        for rule in rules:
            if rule is not None:
                oval = rule.find("./{%s}check[@system=\"%s\"]" %
                                 (xccdf_ns, oval_ns))
                bash_fix = rule.find("./{%s}fix[@system=\"%s\"]" %
                                     (xccdf_ns, bash_rem_system))
                ansible_fix = rule.find("./{%s}fix[@system=\"%s\"]" %
                                        (xccdf_ns, ansible_rem_system))
                puppet_fix = rule.find("./{%s}fix[@system=\"%s\"]" %
                                       (xccdf_ns, puppet_rem_system))
                anaconda_fix = rule.find("./{%s}fix[@system=\"%s\"]" %
                                         (xccdf_ns, anaconda_rem_system))
                cce = rule.find("./{%s}ident[@system=\"%s\"]" %
                                (xccdf_ns, cce_system))
                stig_id = rule.find("./{%s}reference[@href=\"%s\"]" %
                                    (xccdf_ns, stig_ns))

                rule_stats.append(
                    RuleStats(rule.get("id"), oval,
                              bash_fix, ansible_fix, puppet_fix, anaconda_fix,
                              cce, stig_id)
                )

        if not rule_stats:
            print('Unable to retrieve statistics for %s profile' % profile)
            sys.exit(1)

        rule_stats.sort(key=lambda r: r.dict['id'])

        profile_stats['profile_id'] = profile
        if ssg_version_elem is not None:
            profile_stats['ssg_version'] = \
                'SCAP Security Guide %s' % ssg_version_elem.text
        profile_stats['rules_count'] = len(rule_stats)
        profile_stats['implemented_ovals'] = \
            [x.dict['id'] for x in rule_stats if x.dict['oval'] is not None]
        profile_stats['implemented_ovals_pct'] = \
            float(len(profile_stats['implemented_ovals'])) / \
            profile_stats['rules_count'] * 100
        profile_stats['missing_ovals'] = \
            [x.dict['id'] for x in rule_stats if x.dict['oval'] is None]

        profile_stats['implemented_bash_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['bash_fix'] is not None]
        profile_stats['implemented_bash_fixes_pct'] = \
            float(len(profile_stats['implemented_bash_fixes'])) / \
            profile_stats['rules_count'] * 100
        profile_stats['missing_bash_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['bash_fix'] is None]

        profile_stats['implemented_ansible_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['ansible_fix'] is not None]
        profile_stats['implemented_ansible_fixes_pct'] = \
            float(len(profile_stats['implemented_ansible_fixes'])) / \
            profile_stats['rules_count'] * 100
        profile_stats['missing_ansible_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['ansible_fix'] is None]

        profile_stats['implemented_puppet_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['puppet_fix'] is not None]
        profile_stats['implemented_puppet_fixes_pct'] = \
            float(len(profile_stats['implemented_puppet_fixes'])) / \
            profile_stats['rules_count'] * 100
        profile_stats['missing_puppet_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['puppet_fix'] is None]

        profile_stats['implemented_anaconda_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['anaconda_fix'] is not None]

        profile_stats['missing_stig_ids'] = []
        if 'stig' in profile_stats['profile_id']:
            profile_stats['missing_stig_ids'] = \
                [x.dict['id'] for x in rule_stats if x.dict['stig_id'] is None]

        profile_stats['implemented_anaconda_fixes_pct'] = \
            float(len(profile_stats['implemented_anaconda_fixes'])) / \
            profile_stats['rules_count'] * 100
        profile_stats['missing_anaconda_fixes'] = \
            [x.dict['id'] for x in rule_stats if x.dict['anaconda_fix'] is None]

        profile_stats['assigned_cces'] = \
            [x.dict['id'] for x in rule_stats if x.dict['cce'] is not None]
        profile_stats['assigned_cces_pct'] = \
            float(len(profile_stats['assigned_cces'])) / \
            profile_stats['rules_count'] * 100
        profile_stats['missing_cces'] = \
            [x.dict['id'] for x in rule_stats if x.dict['cce'] is None]

        return profile_stats

    def show_profile_stats(self, profile, options):
        """Displays statistics for specific profile"""

        profile_stats = self.get_profile_stats(profile)
        rules_count = profile_stats['rules_count']
        impl_ovals_count = len(profile_stats['implemented_ovals'])
        impl_bash_fixes_count = len(profile_stats['implemented_bash_fixes'])
        impl_ansible_fixes_count = len(profile_stats['implemented_ansible_fixes'])
        impl_puppet_fixes_count = len(profile_stats['implemented_puppet_fixes'])
        impl_anaconda_fixes_count = len(profile_stats['implemented_anaconda_fixes'])
        missing_stig_ids_count = len(profile_stats['missing_stig_ids'])
        impl_cces_count = len(profile_stats['assigned_cces'])

        if options.format == "plain":
            print("\nProfile %s:" % profile)
            print("* rules:            %d" % rules_count)
            print("* checks (OVAL):    %d\t[%d%% complete]" %
                  (impl_ovals_count,
                   profile_stats['implemented_ovals_pct']))

            print("* fixes (bash):     %d\t[%d%% complete]" %
                  (impl_bash_fixes_count,
                   profile_stats['implemented_bash_fixes_pct']))
            print("* fixes (ansible):  %d\t[%d%% complete]" %
                  (impl_ansible_fixes_count,
                   profile_stats['implemented_ansible_fixes_pct']))
            print("* fixes (puppet):   %d\t[%d%% complete]" %
                  (impl_puppet_fixes_count,
                   profile_stats['implemented_puppet_fixes_pct']))
            print("* fixes (anaconda): %d\t[%d%% complete]" %
                  (impl_anaconda_fixes_count,
                   profile_stats['implemented_anaconda_fixes_pct']))

            print("* CCEs:             %d\t[%d%% complete]" %
                  (impl_cces_count,
                   profile_stats['assigned_cces_pct']))

            if options.implemented_ovals and \
               profile_stats['implemented_ovals']:
                print("** Rules of '%s' " % profile +
                      "profile having OVAL check: %d of %d [%d%% complete]" %
                      (impl_ovals_count, rules_count,
                       profile_stats['implemented_ovals_pct']))
                self.console_print(profile_stats['implemented_ovals'],
                                   console_width)

            if options.implemented_fixes:
                if profile_stats['implemented_bash_fixes']:
                    print("*** Rules of '%s' profile having "
                          "a bash fix script: %d of %d [%d%% complete]"
                          % (profile, impl_bash_fixes_count, rules_count,
                             profile_stats['implemented_bash_fixes_pct']))
                    self.console_print(profile_stats['implemented_bash_fixes'],
                                       console_width)

                if profile_stats['implemented_ansible_fixes']:
                    print("*** Rules of '%s' profile having "
                          "a ansible fix script: %d of %d [%d%% complete]"
                          % (profile, impl_ansible_fixes_count, rules_count,
                             profile_stats['implemented_ansible_fixes_pct']))
                    self.console_print(
                        profile_stats['implemented_ansible_fixes'],
                        console_width)

                if profile_stats['implemented_puppet_fixes']:
                    print("*** Rules of '%s' profile having "
                          "a puppet fix script: %d of %d [%d%% complete]"
                          % (profile, impl_puppet_fixes_count, rules_count,
                             profile_stats['implemented_puppet_fixes_pct']))
                    self.console_print(
                        profile_stats['implemented_puppet_fixes'],
                        console_width)

                if profile_stats['implemented_anaconda_fixes']:
                    print("*** Rules of '%s' profile having "
                          "a anaconda fix script: %d of %d [%d%% complete]"
                          % (profile, impl_anaconda_fixes_count, rules_count,
                             profile_stats['implemented_anaconda_fixes_pct']))
                    self.console_print(
                        profile_stats['implemented_anaconda_fixes'],
                        console_width)

            if options.assigned_cces and \
               profile_stats['assigned_cces']:
                print("*** Rules of '%s' " % profile +
                      "profile having CCE assigned: %d of %d [%d%% complete]" %
                      (impl_cces_count, rules_count,
                       profile_stats['assigned_cces_pct']))
                self.console_print(profile_stats['assigned_cces'],
                                   console_width)

            if options.missing_ovals and profile_stats['missing_ovals']:
                print("*** Rules of '%s' " % profile + "profile missing " +
                      "OVAL: %d of %d [%d%% complete]" %
                      (rules_count - impl_ovals_count, rules_count,
                       profile_stats['implemented_ovals_pct']))
                self.console_print(profile_stats['missing_ovals'],
                                   console_width)

            if options.missing_fixes:
                if profile_stats['missing_bash_fixes']:
                    print("*** rules of '%s' profile missing "
                          "a bash fix script: %d of %d [%d%% complete]"
                          % (profile, rules_count - impl_bash_fixes_count,
                             rules_count,
                             profile_stats['implemented_bash_fixes_pct']))
                    self.console_print(profile_stats['missing_bash_fixes'],
                                       console_width)

                if profile_stats['missing_ansible_fixes']:
                    print("*** rules of '%s' profile missing "
                          "a ansible fix script: %d of %d [%d%% complete]"
                          % (profile, rules_count - impl_ansible_fixes_count,
                             rules_count,
                             profile_stats['implemented_ansible_fixes_pct']))
                    self.console_print(profile_stats['missing_ansible_fixes'],
                                       console_width)

                if profile_stats['missing_puppet_fixes']:
                    print("*** rules of '%s' profile missing "
                          "a puppet fix script: %d of %d [%d%% complete]"
                          % (profile, rules_count - impl_puppet_fixes_count,
                             rules_count,
                             profile_stats['implemented_puppet_fixes_pct']))
                    self.console_print(profile_stats['missing_puppet_fixes'],
                                       console_width)

                if profile_stats['missing_anaconda_fixes']:
                    print("*** rules of '%s' profile missing "
                          "a anaconda fix script: %d of %d [%d%% complete]"
                          % (profile, rules_count - impl_anaconda_fixes_count,
                             rules_count,
                             profile_stats['implemented_anaconda_fixes_pct']))
                    self.console_print(profile_stats['missing_anaconda_fixes'],
                                       console_width)

            if options.missing_stig_ids and profile_stats['missing_stig_ids']:
                print("*** rules of '%s' profile missing "
                      "STIG IDs: %d of %d have them [%d%% missing]"
                      % (profile, rules_count - missing_stig_ids_count,
                         rules_count,
                         (100.0 * missing_stig_ids_count / rules_count)))
                self.console_print(profile_stats['missing_stig_ids'],
                                   console_width)

            if options.missing_cces and profile_stats['missing_cces']:
                print("***Rules of '%s' " % profile + "profile missing " +
                      "CCE identifier: %d of %d [%d%% complete]" %
                      (rules_count - impl_cces_count, rules_count,
                       profile_stats['assigned_cces_pct']))
                self.console_print(profile_stats['missing_cces'],
                                   console_width)

        else:
            # First delete the not requested information
            if not options.missing_ovals:
                del profile_stats['missing_ovals']
            if not options.missing_fixes:
                del profile_stats['missing_bash_fixes']
                del profile_stats['missing_ansible_fixes']
                del profile_stats['missing_puppet_fixes']
                del profile_stats['missing_anaconda_fixes']
                del profile_stats['missing_stig_ids']
            if not options.missing_cces:
                del profile_stats['missing_cces']
            if not options.implemented_ovals:
                del profile_stats['implemented_ovals']
            if not options.implemented_fixes:
                del profile_stats['implemented_bash_fixes']
                del profile_stats['implemented_ansible_fixes']
                del profile_stats['implemented_puppet_fixes']
                del profile_stats['implemented_anaconda_fixes']
            if not options.assigned_cces:
                del profile_stats['assigned_cces']

            return profile_stats

    def console_print(self, content, width):
        """Prints the 'content' array left aligned, each time 45 characters
           long, each row 'width' characters wide"""

        msg = ''
        for x in content:
            if len(msg) + len(x) < width - 6:
                msg += '   ' + "%-45s" % x
            else:
                print("%s" % msg)
                msg = '   ' + "%-45s" % x
        if msg != '':
            print("%s" % msg)


def main():
    parser = argparse.ArgumentParser(description=script_desc)
    parser.add_argument("--profile", "-p",
                        action="store",
                        help="Show statistics for this XCCDF Profile only. If "
                        "not provided the script will show stats for all "
                        "available profiles.")
    parser.add_argument("--benchmark", "-b", required=True,
                        action="store",
                        help="Specify XCCDF file to act on. Must be a plain "
                        "XCCDF file, doesn't work on source datastreams yet!")
    parser.add_argument("--implemented-ovals", default=False,
                        action="store_true", dest="implemented_ovals",
                        help="Show IDs of implemented OVAL checks.")
    parser.add_argument("--missing-stig-ids", default=False,
                        action="store_true", dest="missing_stig_ids",
                        help="Show rules in STIG profiles that don't have STIG IDs.")
    parser.add_argument("--missing-ovals", default=False,
                        action="store_true", dest="missing_ovals",
                        help="Show IDs of unimplemented OVAL checks.")
    parser.add_argument("--implemented-fixes", default=False,
                        action="store_true", dest="implemented_fixes",
                        help="Show IDs of implemented remediations.")
    parser.add_argument("--missing-fixes", default=False,
                        action="store_true", dest="missing_fixes",
                        help="Show IDs of unimplemented remediations.")
    parser.add_argument("--assigned-cces", default=False,
                        action="store_true", dest="assigned_cces",
                        help="Show IDs of rules having CCE assigned.")
    parser.add_argument("--missing-cces", default=False,
                        action="store_true", dest="missing_cces",
                        help="Show IDs of rules missing CCE element.")
    parser.add_argument("--implemented", default=False,
                        action="store_true",
                        help="Equivalent of --implemented-ovals, "
                        "--implemented_fixes and --assigned-cves "
                        "all being set.")
    parser.add_argument("--missing", default=False,
                        action="store_true",
                        help="Equivalent of --missing-ovals, --missing-fixes"
                        " and --missing-cces all being set.")
    parser.add_argument("--all", default=False,
                        action="store_true", dest="all",
                        help="Show all available statistics.")
    parser.add_argument("--format", default="plain",
                        choices=["plain", "json", "csv"],
                        help="Which format to use for output.")

    args, unknown = parser.parse_known_args()
    if unknown:
        sys.stderr.write(
            "Unknown positional arguments " + ",".join(unknown) + ".\n"
        )
        sys.exit(1)

    if args.all:
        args.implemented = True
        args.missing = True

    if args.implemented:
        args.implemented_ovals = True
        args.implemented_fixes = True
        args.assigned_cces = True

    if args.missing:
        args.missing_ovals = True
        args.missing_fixes = True
        args.missing_cces = True
        args.missing_stig_ids = True

    benchmark = XCCDFBenchmark(args.benchmark)
    ret = []
    if args.profile:
        ret.append(benchmark.show_profile_stats(args.profile, args))
    else:
        all_profile_elems = benchmark.tree.findall("./{%s}Profile" % (xccdf_ns))
        ret = []
        for elem in all_profile_elems:
            profile = elem.get('id')
            if profile is not None:
                ret.append(benchmark.show_profile_stats(profile, args))

    if args.format == "json":
        print(json.dumps(ret, indent=4))
    elif args.format == "csv":
        # we can assume ret has at least one element
        # CSV header
        print(",".join(ret[0].keys()))
        for line in ret:
            print(",".join([str(value) for value in line.values()]))


if __name__ == '__main__':
    main()



#!/usr/bin/env python2

from __future__ import print_function

import os
import sys
import argparse

templates_dir = os.path.join(os.path.dirname(__file__), "..", "templates")
sys.path.append(templates_dir)
from template_common import ActionType

from create_accounts_password import AccountsPasswordGenerator
from create_kernel_modules_disabled import KernelModulesDisabledGenerator
from create_mounts import MountsGenerator
from create_mount_options import MountOptionsGenerator
from create_package_installed import PackageInstalledGenerator
from create_package_removed import PackageRemovedGenerator
from create_permission import PermissionGenerator
from create_selinux_booleans import SEBoolGenerator
from create_services_disabled import ServiceDisabledGenerator
from create_services_enabled import ServiceEnabledGenerator
from create_sysctl import SysctlGenerator
from create_audit_rules_dac_modification import AuditRulesDacModificationGenerator
from create_audit_rules_unsuccessful_file_modification import AuditRulesUnsuccessfulFileModificationGenerator
from create_audit_rules_file_deletion_events import AuditRulesFileDeletionEventsGenerator
from create_audit_rules_login_events import AuditRulesLoginEventsGenerator
from create_audit_rules_privileged_commands import AuditRulesPrivilegedCommandsGenerator
from create_audit_rules_usergroup_modification import AuditRulesUserGroupModificationGenerator
from create_audit_rules_execution import AuditRulesExecutionGenerator
from create_file_groupowner import FileGroupOwnerGenerator
from create_file_owner import FileOwnerGenerator
from create_file_permissions import FilePermissionsGenerator


class Builder(object):
    def __init__(self):
        self.input_dir = None
        self.output_dir = None
        self.ssg_shared = ""

        self.script_dict = {
            "sysctl_values.csv":                SysctlGenerator(),
            "services_disabled.csv":            ServiceDisabledGenerator(),
            "services_enabled.csv":             ServiceEnabledGenerator(),
            "packages_installed.csv":           PackageInstalledGenerator(),
            "packages_removed.csv":             PackageRemovedGenerator(),
            "kernel_modules_disabled.csv":      KernelModulesDisabledGenerator(),
            "file_dir_permissions.csv":         PermissionGenerator(),
            "accounts_password.csv":            AccountsPasswordGenerator(),
            "mounts.csv":                       MountsGenerator(),
            "mount_options.csv":                MountOptionsGenerator(),
            "selinux_booleans.csv":             SEBoolGenerator(),
            "audit_rules_dac_modification.csv": AuditRulesDacModificationGenerator(),
            "audit_rules_unsuccessful_file_modification.csv":   AuditRulesUnsuccessfulFileModificationGenerator(),
            "audit_rules_file_deletion_events.csv":  AuditRulesFileDeletionEventsGenerator(),
            "audit_rules_login_events.csv":  AuditRulesLoginEventsGenerator(),
            "audit_rules_privileged_commands.csv":  AuditRulesPrivilegedCommandsGenerator(),
            "audit_rules_usergroup_modification.csv":  AuditRulesUserGroupModificationGenerator(),
            "audit_rules_execution.csv":        AuditRulesExecutionGenerator(),
            "file_groupowner.csv":              FileGroupOwnerGenerator(),
            "file_owner.csv":                   FileOwnerGenerator(),
            "file_permissions.csv":             FilePermissionsGenerator(),
        }
        self.supported_ovals = ["oval_5.10"]
        self.langs = ["bash", "ansible", "oval", "anaconda", "puppet"]
        utils_dir = os.path.dirname(os.path.realpath(__file__))
        root_dir = os.path.join(utils_dir, "..", "..")
        self.shared_templates_dir = \
            os.path.join(root_dir, "shared", "templates")

        self.current_oval = "oval_5.10"

    def set_langs(self, langs):
        self.langs = langs

    def set_input_dir(self, input_dir):
        self.input_dir = input_dir
        self.templates_dirs = {
            "oval_5.10": self.input_dir,
            "oval_5.11": os.path.join(self.input_dir, "oval_5.11_templates")
        }

        self.csv_dirs = {
            "oval_5.10": os.path.join(self.input_dir, "csv"),
            "oval_5.11": os.path.join(self.input_dir, "csv", "oval_5.11"),
        }

    def _set_current_oval(self, oval):
        self.current_oval = oval

    def _get_template_dir(self):
        return self.templates_dirs[self.current_oval]

    def _get_csv_dir(self):
        return self.csv_dirs[self.current_oval]

    def _get_csv_list(self):
        dir_ = self._get_csv_dir()

        csvs = []

        try:
            files = os.listdir(dir_)
        except OSError:
            return []

        for file_ in files:
            # skip non csv files
            if not file_.endswith(".csv"):
                continue

            # skip empty files
            filepath = os.path.join(dir_, file_)
            if os.stat(filepath).st_size == 0:
                continue

            csvs.append(file_)

        return csvs

    def _get_generator_for_csv(self, csv_filename):
        try:
            return self.script_dict[csv_filename]

        except KeyError:
            sys.stderr.write(
                "Cannot find the associated generator class for {0}\n"
                .format(csv_filename)
            )
            #sys.exit(1)

    def _deduplicate(self, files):
        return set(os.path.realpath(file_) for file_ in files)

    def build(self):
        for lang in self.langs:
            dir_ = os.path.join(self.output_dir, lang)
            if not os.path.exists(dir_):
                os.makedirs(dir_)

        # Build scripts for multiple OVAL versions.
        # At first for the oldest OVAL, then newer and newer
        # this will allow to override older implementation
        # with a never one.
        for oval in self.supported_ovals:
            self._set_current_oval(oval)

            csv_dir = self._get_csv_dir()
            for csv_filename in self._get_csv_list():
                generator = self._get_generator_for_csv(csv_filename)
                csv_filepath = os.path.join(csv_dir, csv_filename)

                generator.reset()
                generator.output_dir = self.output_dir
                generator.action = ActionType.BUILD
                generator.product_input_dir = self._get_template_dir()
                generator.shared_dir = self.ssg_shared

                for lang in self.langs:
                    generator.csv_map(csv_filepath, language=lang)

    def get_file_list(self, action):
        assert(action in [ActionType.INPUT, ActionType.OUTPUT])

        list_ = []
        if action == ActionType.INPUT:
            pass

        for oval in self.supported_ovals:
            self._set_current_oval(oval)

            csv_dir = self._get_csv_dir()
            for csv in self._get_csv_list():
                csv_filepath = os.path.join(csv_dir, csv)
                generator = self._get_generator_for_csv(csv)

                if action == ActionType.INPUT:
                    list_.append(csv_filepath)

                generator.reset()
                generator.output_dir = self.output_dir
                generator.action = action
                generator.product_input_dir = self._get_template_dir()
                generator.shared_dir = self.ssg_shared

                for lang in self.langs:
                    generator.csv_map(csv_filepath, language=lang)

                list_.extend(generator.files)

        return self._deduplicate(list_)

    def list_inputs(self):
        for file_ in self.get_file_list(ActionType.INPUT):
            print(file_)

    def list_outputs(self):
        for file_ in self.get_file_list(ActionType.OUTPUT):
            print(file_)


if __name__ == "__main__":
    p = argparse.ArgumentParser()

    sp = p.add_subparsers(help="actions")

    make_sp = sp.add_parser('build', help="Build remediations")
    make_sp.set_defaults(cmd="build")

    input_sp = sp.add_parser('list-inputs', help="Generate input list")
    input_sp.set_defaults(cmd="list_inputs")

    output_sp = sp.add_parser('list-outputs', help="Generate output list")
    output_sp.set_defaults(cmd="list_outputs")

    p.add_argument('--language', metavar="LANG", default=None,
                   help="Scripts of which language should we generate? "
                   "Default: all.")
    p.add_argument("-i", "--input", action="store", required=True,
                   help="input directory")
    p.add_argument("-o", "--output", action="store", required=True,
                   help="output directory")
    p.add_argument("-s", "--shared", metavar="PATH", required=True,
                   help="Full absolute path to SSG shared directory")
    p.add_argument('--oval_version', action="store", default="5.10",
                   help="oval version")

    args, unknown = p.parse_known_args()
    if unknown:
        sys.stderr.write(
            "Unknown positional arguments " + ",".join(unknown) + ".\n"
        )
        sys.exit(1)

    builder = Builder()
    if args.language is not None:
        builder.set_langs([args.language])

    builder.set_input_dir(args.input)
    builder.output_dir = args.output
    builder.ssg_shared = args.shared

    if args.oval_version == "5.10":
        builder.supported_ovals = ["oval_5.10"]

    elif args.oval_version == "5.11":
        builder.supported_ovals = ["oval_5.10", "oval_5.11"]

    else:
        sys.stderr.write("Unknown oval version")
        sys.exit(1)

    func = getattr(builder, args.cmd)
    func()



#
# create_package_removed.py
#   automatically generate checks for removed packages

from template_common import FilesGenerator, UnknownTargetError


class PackageRemovedGenerator(FilesGenerator):
    def generate(self, target, package_info):
        pkgname = package_info[0]
        if not pkgname:
            raise RuntimeError(
                "ERROR: input violation: the package name must be defined")

        if target == "oval":
            self.file_from_template(
                "./template_OVAL_package_removed",
                {"%PKGNAME%": pkgname},
                "./oval/package_{0}_removed.xml", pkgname
            )

        elif target == "bash":
            self.file_from_template(
                "./template_BASH_package_removed",
                {"%PKGNAME%": pkgname},
                "./bash/package_{0}_removed.sh", pkgname
            )

        elif target == "ansible":
            self.file_from_template(
                "./template_ANSIBLE_package_removed",
                {"%PKGNAME%": pkgname},
                "./ansible/package_{0}_removed.yml", pkgname
            )

        elif target == "anaconda":
            self.file_from_template(
                "./template_ANACONDA_package_removed",
                {"%PKGNAME%": pkgname},
                "./anaconda/package_{0}_removed.anaconda", pkgname
            )

        elif target == "puppet":
            self.file_from_template(
                "./template_PUPPET_package_removed",
                {"%PKGNAME%": pkgname},
                "./puppet/package_{0}_removed.pp", pkgname
            )

        else:
            raise UnknownTargetError(target)

    def csv_format(self):
        return("CSV should contains lines of the format: " +
               "PACKAGE_NAME")



#!/usr/bin/env python2

from __future__ import print_function

"""
Takes given XCCDF or DataStream and adds RHEL derivative operating system(s) CPE name next
to RHEL CPE names. Can automatically recognize RHEL6, 7, etc. CPEs and adds the derivitive OS ones
next to those accordingly.

Apart from adding the CPEs it adds a notice informing the user that the content
has been enabled for the derivative operating systems and what are the implications.

Author: Martin Preisler <mpreisle@redhat.com>
"""

try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    import cElementTree as ElementTree

import os
import sys
import re
from optparse import OptionParser

# Put shared python modules in path
sys.path.insert(0, os.path.join(
        os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
        "modules"))
import ssgcommon

XCCDF11_NS = ssgcommon.XCCDF11_NS
XCCDF12_NS = ssgcommon.XCCDF12_NS

RHEL_CENTOS_CPE_MAPPING = {
    "cpe:/o:redhat:enterprise_linux:6": "cpe:/o:centos:centos:6",
    "cpe:/o:redhat:enterprise_linux:7": "cpe:/o:centos:centos:7",
}

RHEL_SL_CPE_MAPPING = {
    "cpe:/o:redhat:enterprise_linux:6": "cpe:/o:scientificlinux:scientificlinux:6",
    "cpe:/o:redhat:enterprise_linux:7": "cpe:/o:scientificlinux:scientificlinux:7",
}

CENTOS_NOTICE = \
    "<div xmlns=\"http://www.w3.org/1999/xhtml\">\n" \
    "<p>This benchmark is a direct port of a <i>SCAP Security Guide </i> " \
    "benchmark developed for <i>Red Hat Enterprise Linux</i>. It has been " \
    "modified through an automated process to remove specific dependencies " \
    "on <i>Red Hat Enterprise Linux</i> and to function with <i>CentOS</i>. " \
    "The result is a generally useful <i>SCAP Security Guide</i> benchmark " \
    "with the following caveats:</p>\n" \
    "<ul>\n" \
    "<li><i>CentOS</i> is not an exact copy of " \
    "<i>Red Hat Enterprise Linux</i>. There may be configuration differences " \
    "that produce false positives and/or false negatives. If this occurs " \
    "please file a bug report.</li>\n" \
    "\n" \
    "<li><i>CentOS</i> has its own build system, compiler options, patchsets, " \
    "and is a community supported, non-commercial operating system. " \
    "<i>CentOS</i> does not inherit " \
    "certifications or evaluations from <i>Red Hat Enterprise Linux</i>. As " \
    "such, some configuration rules (such as those requiring " \
    "<i>FIPS 140-2</i> encryption) will continue to fail on <i>CentOS</i>.</li>\n" \
    "</ul>\n" \
    "\n" \
    "<p>Members of the <i>CentOS</i> community are invited to participate in " \
    "<a href=\"http://open-scap.org\">OpenSCAP</a> and " \
    "<a href=\"https://github.com/OpenSCAP/scap-security-guide\">" \
    "SCAP Security Guide</a> development. Bug reports and patches " \
    "can be sent to GitHub: " \
    "<a href=\"https://github.com/OpenSCAP/scap-security-guide\">" \
    "https://github.com/OpenSCAP/scap-security-guide</a>. " \
    "The mailing list is at " \
    "<a href=\"https://fedorahosted.org/mailman/listinfo/scap-security-guide\">" \
    "https://fedorahosted.org/mailman/listinfo/scap-security-guide</a>" \
    ".</p>" \
    "</div>"

SL_NOTICE = \
    "<div xmlns=\"http://www.w3.org/1999/xhtml\">\n" \
    "<p>This benchmark is a direct port of a <i>SCAP Security Guide </i> " \
    "benchmark developed for <i>Red Hat Enterprise Linux</i>. It has been " \
    "modified through an automated process to remove specific dependencies " \
    "on <i>Red Hat Enterprise Linux</i> and to function with <i>Scientifc Linux</i>. " \
    "The result is a generally useful <i>SCAP Security Guide</i> benchmark " \
    "with the following caveats:</p>\n" \
    "<ul>\n" \
    "<li><i>Scientifc Linux</i> is not an exact copy of " \
    "<i>Red Hat Enterprise Linux</i>. Scientific Linux is a Linux distribution " \
    "produced by <i>Fermi National Accelerator Laboratory</i>. It is a free and " \
    "open source operating system based on <i>Red Hat Enterprise Linux</i> and aims " \
    "to be \"as close to the commercial enterprise distribution as we can get it.\" " \
    "There may be configuration differences that produce false positives and/or " \
    "false negatives. If this occurs please file a bug report.</li>\n" \
    "\n" \
    "<li><i>Scientifc Linux</i> is derived from the free and open source software " \
    "made available by Red Hat, but it is not produced, maintained or supported by <i>Red Hat</i>. " \
    "<i>Scientifc Linux</i> has its own build system, compiler options, patchsets, " \
    "and is a community supported, non-commercial operating system. " \
    "<i>Scientifc Linux</i> does not inherit " \
    "certifications or evaluations from <i>Red Hat Enterprise Linux</i>. As " \
    "such, some configuration rules (such as those requiring " \
    "<i>FIPS 140-2</i> encryption) will continue to fail on <i>Scientifc Linux</i>.</li>\n" \
    "</ul>\n" \
    "\n" \
    "<p>Members of the <i>Scientifc Linux</i> community are invited to participate in " \
    "<a href=\"http://open-scap.org\">OpenSCAP</a> and " \
    "<a href=\"https://github.com/OpenSCAP/scap-security-guide\">" \
    "SCAP Security Guide</a> development. Bug reports and patches " \
    "can be sent to GitHub: " \
    "<a href=\"https://github.com/OpenSCAP/scap-security-guide\">" \
    "https://github.com/OpenSCAP/scap-security-guide</a>. " \
    "The mailing list is at " \
    "<a href=\"https://fedorahosted.org/mailman/listinfo/scap-security-guide\">" \
    "https://fedorahosted.org/mailman/listinfo/scap-security-guide</a>" \
    ".</p>" \
    "</div>"

CENTOS_NOTICE_ELEMENT = ElementTree.fromstring(CENTOS_NOTICE)
SL_NOTICE_ELEMENT = ElementTree.fromstring(SL_NOTICE)

CENTOS_WARNING = 'centos_warning'
SL_WARNING = 'sl_warning'


def add_derivative_cpes(elem, namespace, mapping):
    """Adds derivative CPEs next to RHEL ones, checks XCCDF elements of given
    namespace.
    """

    affected = False

    for child in list(elem):
        affected = affected or add_derivative_cpes(child, namespace, mapping)

    # precompute this so that we can affect the tree while iterating
    children = list(elem.findall(".//{%s}platform" % (namespace)))

    for child in children:
        idref = child.get("idref")
        if idref in mapping:
            new_platform = ElementTree.Element("{%s}platform" % (namespace))
            new_platform.set("idref", mapping[idref])
            # this is done for the newline and indentation
            new_platform.tail = child.tail

            index = list(elem).index(child)
            # insert it right after the respective RHEL CPE
            elem.insert(index + 1, new_platform)

            affected = True

    return affected


def add_derivative_notice(benchmark, namespace, notice, warning):
    """Adds derivative notice as the first notice to given benchmark.
    """

    index = -1
    prev_element = None
    existing_notices = list(benchmark.findall("./{%s}notice" % (namespace)))
    if len(existing_notices) > 0:
        prev_element = existing_notices[0]
        # insert before the first notice
        index = list(benchmark).index(prev_element)
    else:
        existing_descriptions = list(
            benchmark.findall("./{%s}description" % (namespace))
        )
        prev_element = existing_descriptions[-1]
        # insert after the last description
        index = list(benchmark).index(prev_element) + 1

    if index == -1:
        raise RuntimeError(
            "Can't find existing notices or description in benchmark '%s'." %
            (benchmark)
        )

    elem = ElementTree.Element("{%s}notice" % (namespace))
    elem.set("id", warning)
    elem.append(notice)
    # this is done for the newline and indentation
    elem.tail = prev_element.tail
    benchmark.insert(index, elem)

    return True


def remove_rh_idents(tree_root, namespace):
    for rule in tree_root.findall(".//{%s}Rule" % (namespace)):
        for ident in rule.findall(".//{%s}ident" % (namespace)):
            if ident is not None:
                if re.search('CCE-*', ident.text) or re.search('.*RHEL-*', ident.text):
                    rule.remove(ident)

        for ref in rule.findall(".//{%s}reference" % (namespace)):
            if ref.text is not None:
                if re.search('RHEL-*', ref.text):
                    rule.remove(ref)


def main():
    usage = "usage: %prog [options]"
    parser = OptionParser(usage=usage)
    parser.add_option("--enable-centos", dest="centos", default=False,
                      action="store_true", help="Enable CentOS")
    parser.add_option("--enable-sl", dest="sl", default=False,
                      action="store_true", help="Enable Scientific Linux")
    parser.add_option("-i", "--input", dest="input_content", default=False,
                      action="store",
                      help="INPUT can be XCCDF or Source DataStream")
    parser.add_option("-o", "--output", dest="output", default=False,
                      action="store", help="XML Tree content")
    (options, args) = parser.parse_args()

    if options.centos and options.sl:
        sys.stderr.write(
            "Cannot enable two derivative OS(s) at the same time\n"
        )
        parser.print_help()
        sys.exit(1)

    if not options.output and not options.input_content:
        parser.print_help()
        sys.exit(1)

    if options.centos:
        mapping = RHEL_CENTOS_CPE_MAPPING
        notice = CENTOS_NOTICE_ELEMENT
        warning = CENTOS_WARNING

    if options.sl:
        mapping = RHEL_SL_CPE_MAPPING
        notice = SL_NOTICE_ELEMENT
        warning = SL_WARNING

    tree = ElementTree.ElementTree()
    tree.parse(options.input_content)

    root = tree.getroot()

    benchmarks = []

    def scrape_benchmarks(root_element, namespace, dest):
        dest.extend([
            (namespace, elem)
            for elem in list(root.findall(".//{%s}Benchmark" % (namespace)))
        ])
        if root_element.tag == "{%s}Benchmark" % (namespace):
            dest.append((namespace, root_element))

    scrape_benchmarks(root, XCCDF11_NS, benchmarks)
    scrape_benchmarks(root, XCCDF12_NS, benchmarks)

    # Remove CCEs and DISA STIG IDs from derivatives as these are specific to
    # the vendor/OS.
    remove_rh_idents(root, XCCDF11_NS)
    remove_rh_idents(root, XCCDF12_NS)

    if len(benchmarks) == 0:
        raise RuntimeError("No Benchmark found!")

    for namespace, benchmark in benchmarks:
        if not add_derivative_cpes(benchmark, namespace, mapping):
            raise RuntimeError(
                "Could not add derivative OS CPEs to Benchmark '%s'."
                % (benchmark)
            )

        if not add_derivative_notice(benchmark, namespace, notice, warning):
            raise RuntimeError(
                "Managed to add derivative OS CPEs but failed to add the "
                "notice to affected XCCDF Benchmark '%s'." % (benchmark)
            )

    tree.write(options.output)


if __name__ == "__main__":
    main()



#!/usr/bin/env python2

import sys
import os

# always use shared/testoval_module.py version
script_directory = os.path.dirname(os.path.realpath(__file__))
SHARED_MODULE_PATH = os.path.join(script_directory, "..", "modules")

sys.path.insert(0, SHARED_MODULE_PATH)
import testoval_module

if __name__ == "__main__":
    testoval_module.main()



#!/usr/bin/env python2

import tempfile
import sys
import os.path


# Put shared python modules in path
sys.path.insert(0, os.path.join(
        os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
        "modules"))
import ssgcommon


svg_benchmark = """<?xml version="1.0"?>
<Benchmark xmlns="http://checklists.nist.gov/xccdf/1.1"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
           id="RHEL-6" resolved="1" xml:lang="en-US">
  <status date="2014-08-04">draft</status>
  <title xmlns:xhtml="http://www.w3.org/1999/xhtml" xml:lang="en-US"/>
  <description xmlns:xhtml="http://www.w3.org/1999/xhtml" xml:lang="en-US"/>
  <notice xmlns:xhtml="http://www.w3.org/1999/xhtml" xml:lang="en-US" id="terms_of_use"/>
  <front-matter xmlns:xhtml="http://www.w3.org/1999/xhtml" xml:lang="en-US">
    <p xmlns="http://www.w3.org/1999/xhtml">
      <svg xmlns="http://www.w3.org/2000/svg"
           xmlns:xlink="http://www.w3.org/1999/xlink"
           width="100px" height="100px">
        <circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="darkred" />
      </svg>
    </p>
  </front-matter>
  <platform idref="cpe:/o:redhat:enterprise_linux:6"/>
  <version>0.9</version>
  <model system="urn:xccdf:scoring:default"/>
</Benchmark>
"""


def main():
    oscap_executable = sys.argv[1]

    xccdf = tempfile.NamedTemporaryFile()
    xccdf.write(svg_benchmark.encode("utf-8"))
    xccdf.flush()

    out = ssgcommon.subprocess_check_output(
        [oscap_executable, "xccdf", "generate", "guide", xccdf.name]
    ).decode("utf-8")

    # check whether oscap threw away the SVG elements
    sys.exit(0 if "circle" in out else 1)


if __name__ == "__main__":
    main()



import re

from template_common import FilesGenerator, UnknownTargetError


class SysctlGenerator(FilesGenerator):
    def is_ipv6_id(self, var_id):
        return var_id.find("ipv6") >= 0

    def get_files_var_for_id(self, var_id):

        if self.is_ipv6_id(var_id):
            template_name = 'template_OVAL_sysctl_ipv6'
        else:
            template_name = 'template_OVAL_sysctl'

        return {
            'template_OVAL_sysctl_static_var': 'sysctl_static_',
            'template_OVAL_sysctl_runtime_var': 'sysctl_runtime_',
            template_name: 'sysctl_'
        }

    def get_files_for_id(self, var_id):
        if self.is_ipv6_id(var_id):
            template_name = 'template_OVAL_sysctl_ipv6'
        else:
            template_name = 'template_OVAL_sysctl'

        return {
            'template_OVAL_sysctl_static': 'sysctl_static_',
            'template_OVAL_sysctl_runtime': 'sysctl_runtime_',
            template_name: 'sysctl_'
        }

    def generate(self, target, serviceinfo):
        # get the items out of the list
        sysctl_var, sysctl_val = serviceinfo
        # convert variable name to a format suitable for 'id' tags
        sysctl_var_id = re.sub('[-\.]', '_', sysctl_var)

        if target == "bash":
            # if the sysctl value is not present, use the variable template
            if not sysctl_val.strip():
                # open the template and perform the conversions
                self.file_from_template(
                    "template_BASH_sysctl_var",
                    {
                        "%SYSCTLID%":  sysctl_var_id,
                        "%SYSCTLVAR%": sysctl_var
                    },
                    "./bash/sysctl_{0}.sh", sysctl_var_id
                )
            else:
                self.file_from_template(
                    "./template_BASH_sysctl",
                    {
                        "%SYSCTLID%":  sysctl_var_id,
                        "%SYSCTLVAR%": sysctl_var,
                        "%SYSCTLVAL%": sysctl_val
                    },
                    "./bash/sysctl_{0}.sh", sysctl_var_id
                )

        elif target == "ansible":
            # if the sysctl value is not present, use the variable template
            if not sysctl_val.strip():
                # open the template and perform the conversions
                self.file_from_template(
                    "template_ANSIBLE_sysctl_var",
                    {
                        "%SYSCTLID%":  sysctl_var_id,
                        "%SYSCTLVAR%": sysctl_var
                    },
                    "./ansible/sysctl_{0}.yml", sysctl_var_id
                )
            else:
                self.file_from_template(
                    "./template_ANSIBLE_sysctl",
                    {
                        "%SYSCTLID%":  sysctl_var_id,
                        "%SYSCTLVAR%": sysctl_var,
                        "%SYSCTLVAL%": sysctl_val
                    },
                    "./ansible/sysctl_{0}.yml", sysctl_var_id
                )

        elif target == "oval":
            # if the sysctl value is not present, use the variable template
            if not sysctl_val.strip():

                # open the template files and perform the conversions
                for sysctlfile, prefix in self.get_files_var_for_id(sysctl_var_id).items():
                    self.file_from_template(
                        sysctlfile,
                        {
                            "%SYSCTLID%":  sysctl_var_id,
                            "%SYSCTLVAR%": sysctl_var,
                        },
                        "./oval/{0}.xml", prefix + sysctl_var_id
                    )

            else:
                # open the template files and perform the conversions
                for sysctlfile, prefix in self.get_files_for_id(sysctl_var_id).items():
                    self.file_from_template(
                        sysctlfile,
                        {
                            "%SYSCTLID%":  sysctl_var_id,
                            "%SYSCTLVAR%": sysctl_var,
                            "%SYSCTLVAL%": sysctl_val
                        },
                        "./oval/{0}.xml", prefix + sysctl_var_id
                    )

        else:
            raise UnknownTargetError(target)

    def csv_format(self):
        return("CSV should contains lines of the format: " +
               "sysctlvariable,sysctlvalue")



#!/usr/bin/env python2

import sys
import os
import os.path
import argparse
import codecs

try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    import cElementTree as ElementTree


def preprocess_source(filepath):
    raw = ""
    with codecs.open(filepath, "r", encoding="utf-8") as f:
        raw = f.read()

    source = ""
    for line in raw.splitlines(True):
        if line.startswith("source ") and line.endswith(".sh\n"):
            included_file = line.strip()[7:]  # skip "source "
            with codecs.open(os.path.join(
                os.path.dirname(filepath), included_file), "r", encoding="utf-8"
            ) as f:
                source += f.read()
                source += "\n"
        else:
            source += line

    return source


def main():
    p = argparse.ArgumentParser()
    p.add_argument("--input_dir", required=True)
    p.add_argument("--output", type=argparse.FileType("wb"), required=True)

    args, unknown = p.parse_known_args()
    if unknown:
        sys.stderr.write(
            "Unknown positional arguments " + ",".join(unknown) + ".\n"
        )
        sys.exit(1)

    root = ElementTree.Element("Group")
    root.set("id", "remediation_functions")
    title = ElementTree.SubElement(root, "title")
    title.text = "Remediation functions used by the SCAP Security Guide Project"
    description = ElementTree.SubElement(root, "description")
    description.text = "XCCDF form of the various remediation functions as " \
        "used by remediation scripts from the SCAP Security Guide Project."

    for file_ in os.listdir(args.input_dir):
        if not file_.endswith(".sh"):
            sys.stderr.write(
                "File '%s' does not appear to be a bash script. Skipping!"
                % (file_)
            )
            continue

        filename, ext = os.path.splitext(file_)

        source = preprocess_source(os.path.join(args.input_dir, file_))

        value_element = ElementTree.SubElement(root, "Value")
        value_element.set("id", "function_%s" % (filename))
        value_element.set("type", "string")
        value_element.set("operator", "equals")
        value_element.set("interactive", "0")
        value_element.set("hidden", "true")
        value_element.set("prohibitChanges", "true")

        title = ElementTree.SubElement(value_element, "title")
        title.text = "Remediation function %s" % (filename)

        desc = ElementTree.SubElement(value_element, "description")
        desc.text = "Shared bash remediation function. Not intended to be " \
            "changed by tailoring."

        inner_value = ElementTree.SubElement(value_element, "value")
        inner_value.set("selector", "")
        inner_value.text = source

    tree = ElementTree.ElementTree(root)
    tree.write(args.output)


if __name__ == "__main__":
    main()



"""
A couple generic XCCDF utilities used by build-all-guides.py and
build-all-remediation-roles.py

Author: Martin Preisler <mpreisle@redhat.com>
"""

import re
import ssgcommon

XCCDF11_NS = ssgcommon.XCCDF11_NS
XCCDF12_NS = ssgcommon.XCCDF12_NS

# if a profile ID ends with a string listed here we skip it
PROFILE_ID_BLACKLIST = ["test", "index", "default"]
# filler XCCDF 1.2 prefix which we will strip to avoid very long filenames
PROFILE_ID_PREFIX = ("^xccdf_org.*content_profile_")


def get_benchmark_ids_titles_for_input(input_tree):
    ret = {}

    def scrape_benchmarks(root_element, namespace, dest):
        candidates = \
            list(root_element.findall(".//{%s}Benchmark" % (namespace)))
        if root_element.tag == "{%s}Benchmark" % (namespace):
            candidates.append(root_element)

        for elem in candidates:
            id_ = elem.get("id")
            if id_ is None:
                continue

            title = "<unknown>"
            for element in elem.findall("{%s}title" % (namespace)):
                title = element.text
                break

            dest[id_] = title

    input_root = input_tree.getroot()

    scrape_benchmarks(
        input_root, XCCDF11_NS, ret
    )
    scrape_benchmarks(
        input_root, XCCDF12_NS, ret
    )

    return ret


def get_profile_choices_for_input(input_tree, benchmark_id, tailoring_tree):
    """Returns a dictionary that maps profile_ids to their respective titles.
    """

    # Ideally oscap would have a command line to do this, but as of now it
    # doesn't so we have to implement it ourselves. Importing openscap Python
    # bindings is nasty and overkill for this.

    ret = {}

    def scrape_profiles(root_element, namespace, dest):
        candidates = \
            list(root_element.findall(".//{%s}Benchmark" % (namespace)))
        if root_element.tag == "{%s}Benchmark" % (namespace):
            candidates.append(root_element)

        for benchmark in candidates:
            if benchmark.get("id") != benchmark_id:
                continue

            for elem in benchmark.findall(".//{%s}Profile" % (namespace)):
                id_ = elem.get("id")
                if id_ is None:
                    continue

                title = "<unknown>"
                for element in elem.findall("{%s}title" % (namespace)):
                    title = element.text
                    break

                dest[id_] = title

    input_root = input_tree.getroot()

    scrape_profiles(
        input_root, XCCDF11_NS, ret
    )
    scrape_profiles(
        input_root, XCCDF12_NS, ret
    )

    if tailoring_tree is not None:
        tailoring_root = tailoring_tree.getroot()

        scrape_profiles(
            tailoring_root, XCCDF11_NS, ret
        )
        scrape_profiles(
            tailoring_root, XCCDF12_NS, ret
        )

    return ret


def get_profile_short_id(long_id):
    """If given profile ID is the XCCDF 1.2 long ID this function shortens it
    """

    if re.search(PROFILE_ID_PREFIX, long_id):
        return long_id[re.search(PROFILE_ID_PREFIX, long_id).end():]

    return long_id



#!/usr/bin/env python2

from __future__ import print_function

from tempfile import mkdtemp
import os
import os.path
import sys
import shutil
import re
import argparse
import getpass
import yaml

try:
    from github import Github, InputGitAuthor
except ImportError:
    sys.stderr.write("Please install PyGithub, on Fedora it's in the "
                     "python-PyGithub package.\n")
    sys.exit(1)


# Put shared python modules in path
sys.path.insert(0, os.path.join(
        os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
        "modules"))
import ssgcommon

ORGANIZATION_NAME = "Ansible-Security-Compliance"
GIT_COMMIT_AUTHOR_NAME = "SCAP Security Guide development team"
GIT_COMMIT_AUTHOR_EMAIL = "scap-security-guide@lists.fedorahosted.org"
META_TEMPLATE_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "ansible_galaxy_meta_template.yml"
)
README_TEMPLATE_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "ansible_galaxy_readme_template.md"
)


def create_empty_repositories(github_new_repos, github_org):
    for github_new_repo in github_new_repos:
        print("Creating new Github repository: %s" % github_new_repo)
        github_org.create_repo(
            github_new_repo,
            description="Role generated from SCAP Security Guide",
            homepage="https://www.open-scap.org/",
            private=False,
            has_issues=False,
            has_wiki=False,
            has_downloads=False)


def clone_and_init_repository(parent_dir, repo):
    os.system(
        "git clone git@github.com:%s/%s.git" % (ORGANIZATION_NAME, repo))
    os.system("ansible-galaxy init " + repo + " --force")
    os.chdir(repo)
    try:
        os.system('git add .')
        os.system('git commit -a -m "Initial commit" --author "%s <%s>"'
                  % (GIT_COMMIT_AUTHOR_NAME, GIT_COMMIT_AUTHOR_EMAIL))
        os.system('git push origin master')
    finally:
        os.chdir("..")


def update_repository(repository, local_file_path):
    print("Processing %s..." % repository.name)

    with open(local_file_path, 'r') as f:
        filedata = f.read()

    role_data = yaml.load(filedata)
    vars_data = []
    if "vars" in role_data[0]:
        vars_data = role_data[0]["vars"]

    tasks_data = []
    if "tasks" in role_data[0]:
        tasks_data = role_data[0]["tasks"]

    # ansible language doesn't allow pre_tasks for roles, if the only pre task
    # is the ansible version check we can ignore it because the minimal version
    # is in role metadata
    if "pre_tasks" in role_data[0]:
        pre_tasks_data = role_data[0]["pre_tasks"]
        if len(pre_tasks_data) == 1 and \
                pre_tasks_data[0]["name"] == \
                ssgcommon.ansible_version_requirement_pre_task_name:
            pass
        else:
            sys.stderr.write(
                "%s contains pre_tasks other than the version check. pre_tasks "
                "are not supported for ansible roles and will be skipped!.\n")

    tasks_local_content = yaml.dump(tasks_data, width=120, indent=4,
                                    default_flow_style=False)
    tasks_remote_content = repository.get_file_contents("/tasks/main.yml")

    if tasks_local_content != tasks_remote_content.decoded_content:
        repository.update_file(
            "/tasks/main.yml",
            "Updates tasks/main.yml",
            tasks_local_content,
            tasks_remote_content.sha,
            author=InputGitAuthor(
                GIT_COMMIT_AUTHOR_NAME, GIT_COMMIT_AUTHOR_EMAIL)
        )

        print("Updating tasks/main.yml in %s" % repository.name)

    vars_local_content = yaml.dump(vars_data, width=120, indent=4,
                                   default_flow_style=False)
    vars_remote_content = repository.get_file_contents("/vars/main.yml")

    if vars_local_content != vars_remote_content.decoded_content:
        repository.update_file(
            "/vars/main.yml",
            "Updates vars/main.yml",
            vars_local_content,
            vars_remote_content.sha,
            author=InputGitAuthor(
                GIT_COMMIT_AUTHOR_NAME, GIT_COMMIT_AUTHOR_EMAIL)
        )

        print("Updating vars/main.yml in %s" % repository.name)

    separator = "#" * 79
    first_separator_pos = filedata.find(separator)
    second_separator_pos = filedata.find(separator,
                                         first_separator_pos + len(separator))
    description = filedata[first_separator_pos + len(separator) + 3:
                           second_separator_pos - 3]
    description = description.replace('# ', '')
    description = description.replace('#', '')
    title = re.search('Profile Title:\s+(.+)$',
                      description, re.MULTILINE).group(1)
    description = description.replace('\n', '  \n')

    repository.edit(
        repository.name,
        description="%s - Ansible role generated from the SCAP Security Guide "
        "project" % (title),
        homepage="https://www.open-scap.org/",
    )

    with open(README_TEMPLATE_PATH, 'r') as f:
        readme_template = f.read()

    local_readme_content = readme_template.replace("@DESCRIPTION@", description)
    local_readme_content = local_readme_content.replace("@TITLE@", title)
    local_readme_content = local_readme_content.replace(
        "@MIN_ANSIBLE_VERSION@", ssgcommon.min_ansible_version)
    local_readme_content = local_readme_content.replace("@ROLE_NAME@",
                                                        repository.name)

    remote_readme_file = repository.get_file_contents("/README.md")

    if local_readme_content != remote_readme_file.decoded_content:
        print("Updating README.md in %s" % repository.name)

        repository.update_file(
            "/README.md",
            "Updates README.md",
            local_readme_content,
            remote_readme_file.sha,
            author=InputGitAuthor(
                GIT_COMMIT_AUTHOR_NAME, GIT_COMMIT_AUTHOR_EMAIL)
        )

    with open(META_TEMPLATE_PATH, 'r') as f:
        meta_template = f.read()

    local_meta_content = meta_template.replace("@DESCRIPTION@", title)
    local_meta_content = local_meta_content.replace(
        "@MIN_ANSIBLE_VERSION@", ssgcommon.min_ansible_version)
    remote_meta_file = repository.get_file_contents("/meta/main.yml")

    if local_meta_content != remote_meta_file.decoded_content:
        print("Updating meta/main.yml in %s" % repository.name)
        repository.update_file(
            "/meta/main.yml",
            "Updates meta/main.yml",
            local_meta_content,
            remote_meta_file.sha,
            author=InputGitAuthor(
                GIT_COMMIT_AUTHOR_NAME, GIT_COMMIT_AUTHOR_EMAIL)
        )


def main():
    parser = argparse.ArgumentParser(
        description='Updates SSG Galaxy Ansible Roles')
    parser.add_argument(
        "--build-roles-dir", required=True,
        help="Path to directory containing the ssg generated roles. Most "
        "likely this is going to be scap-security-guide/build/roles",
        dest="build_roles_dir")
    args = parser.parse_args()

    role_whitelist = set([
        "rhel7-role-C2S",
        "rhel7-role-hipaa",
        "rhel7-role-nist-800-171-cui",
        "rhel7-role-ospp",
        "rhel7-role-pci-dss",
        "rhel7-role-rht-ccp",
        "rhel7-role-stig-rhel7-disa"
    ])

    # the first 4 cut chars are for "ssg-"
    # the last 4 cut chars are for ".yml"
    available_roles = set(
        [f[4:-4]
         for f in os.listdir(args.build_roles_dir) if f.endswith(".yml")]
    )
    # print(available_roles)
    roles = available_roles.intersection(role_whitelist)

    print("Input your GitHub credentials:")
    username = raw_input("username or token: ")
    password = getpass.getpass("password (or empty for token): ")

    github = Github(username, password)
    github_org = github.get_organization(ORGANIZATION_NAME)
    github_repositories = [repo.name for repo in github_org.get_repos()]

    # Create empty repositories
    github_new_repos = sorted(list(set(roles) - set(github_repositories)))
    if github_new_repos:
        create_empty_repositories(github_new_repos, github_org)
        github_repositories = [repo.name for repo in github_org.get_repos()]

        # Locally clone and init repositories
        temp_dir = mkdtemp()
        current_dir = os.getcwd()
        os.chdir(temp_dir)
        try:
            for repo in github_new_repos:
                clone_and_init_repository(temp_dir, repo)
        finally:
            os.chdir(current_dir)
            shutil.rmtree(temp_dir)

    # Update repositories
    for repo in sorted(github_org.get_repos(), key=lambda repo: repo.name):
        if repo.name in roles:
            update_repository(
                repo, os.path.join(args.build_roles_dir,
                                   "ssg-" + repo.name + ".yml")
            )
        else:
            print("Repo %s should be deleted, please verify and do that "
                  "manually!" % repo.name)


if __name__ == "__main__":
    main()



"""
Common methods for generating files from templates
"""

import csv
import sys
import os
import re
from abc import abstractmethod


class ActionType:
    INPUT = 1
    OUTPUT = 2
    BUILD = 3


class UnknownTargetError(ValueError):
    def __init__(self, msg):
        super(UnknownTargetError, self).__init__(
            "Unknown target language: \"{0}\"".format(msg)
        )


class TemplateNotFoundError(RuntimeError):
    def __init__(self, template, paths):
        super(TemplateNotFoundError, self).__init__(
            "Template not found: '%s'. Looked in %s."
            % (template, ", ".join(paths))
        )


TARGET_REGEX = re.compile(r"#\s*only-for:([\s\w,]*)")


class FilesGenerator(object):
    def __init__(self):
        self.delimiter = ','
        self.reset()

    def reset(self):
        self.files = []

    def get_template_filename(self, filename):
        template_filename = os.path.join(self.product_input_dir, filename)

        if os.path.isfile(template_filename):
            return template_filename

        paths = [self.product_input_dir]

        if self.product_input_dir.endswith("oval_5.11_templates"):
            template_filename = os.path.join(
                os.path.dirname(self.product_input_dir), filename
            )
            if os.path.isfile(template_filename):
                return template_filename
            paths.append(os.path.dirname(template_filename))

        shared_template = os.path.join(self.shared_dir, "templates", filename)
        if os.path.isfile(shared_template):
            return shared_template
        paths.append(os.path.dirname(shared_template))

        raise TemplateNotFoundError(filename, paths)

    def load_modified(self, filename, constants_dict, regex_replace=[]):
        """
        Load file and replace constants according to constants_dict and
        regex_replace

        constants_dict: dict of constants - replace ( key -> value)
        regex_dict: dict of regex substitutions - sub ( key -> value)
        """

        template_filename = self.get_template_filename(filename)

        if self.action == ActionType.OUTPUT:
            return ""

        if self.action == ActionType.INPUT:
            self.files.append(template_filename)
            return ""

        with open(template_filename, "r") as template_file:
            filestring = template_file.read()

        for key, value in constants_dict.items():
            if not key.startswith("%") or not key.endswith("%"):
                raise RuntimeError(
                    "Refuse to replace '%s' because it doesn't start and end "
                    "with the %% character. Please follow conventions! "
                    "Class name: %s" % (key, self.__class__))

            filestring = filestring.replace(key, value)

            trimmed_key = key[1:-1]  # the key without the % padding chars
            if trimmed_key in filestring:
                highlighted_filestring = filestring.replace(
                    trimmed_key, "--->%s<---" % (trimmed_key)
                )
                raise RuntimeError(
                    "Trimmed key '%s' was found in the filestring after the "
                    "substitution was performed. This is usually a typo or a "
                    "mistake in the template, the python generator or both. "
                    "In the rare case where this is expected please rename the "
                    "key to something unambiguous. Class name: %s. Filestring "
                    "with the trimmed key highlighted:\n%s"
                    % (trimmed_key, self.__class__, highlighted_filestring))

        for pattern, replacement in regex_replace:
            filestring = re.sub(pattern, replacement, filestring)

        return filestring

    def save_modified(self, filename_format, filename_value, string):
        """
        Save string to file
        """

        if self.action == ActionType.INPUT:
            return

        filename = os.path.join(
            self.output_dir, filename_format.format(filename_value)
        )

        if self.action == ActionType.OUTPUT:
            self.files.append(filename)
            return

        with open(filename, "w") as f:
            f.write(string)

    def file_from_template(self, template_filename, constants,
                           filename_format, filename_value, regex_replace=[]):
        """
        Load template, fill constant and create new file
        @param regex_replace: array of tuples (pattern, replacement)
        """

        try:
            filled_template = \
                self.load_modified(template_filename, constants, regex_replace)

            self.save_modified(filename_format, filename_value, filled_template)

        except TemplateNotFoundError as e:
            if self.action == ActionType.BUILD:
                sys.stderr.write(str(e) + "\n")

    def process_csv_line(self, line, target):
        """
        Remove comments
        Remove line if target is unsupported
        """

        if target is not None:
            match = TARGET_REGEX.search(line)

            if match:
                # if line contains restriction to target, check it
                supported_targets = \
                    [x.strip() for x in match.group(1).split(",")]
                if target not in supported_targets:
                    return None

        # get part before comment
        return (line.split("#")[0]).strip()

    def filter_out_csv_lines(self, csv_file, language):
        """Filter out not applicable lines
        """

        for line in csv_file:
            processed_line = self.process_csv_line(line, language)

            if not processed_line:
                continue

            yield processed_line

    def csv_map(self, filename, language):
        """Call specified function on every line of file

        CSV lines can look like:
            col1, col2 # comment
            col3, col4 # only-for: bash, oval
        """

        with open(filename, "r") as csv_file:
            filtered_file = self.filter_out_csv_lines(
                csv_file.readlines(), language)
            csv_lines_content = csv.reader(filtered_file,
                                           delimiter=self.delimiter)

            try:
                for csv_line in csv_lines_content:
                    self.generate(language, csv_line)

            except UnknownTargetError as e:
                if self.action == ActionType.BUILD:
                    sys.stderr.write(str(e) + "\n")

    @abstractmethod
    def csv_format(self):
        raise NotImplementedError("Please Implement this method")



#
# create_permission.py
#        generate template-based checks for file permissions/ownership

import re

from template_common import FilesGenerator, UnknownTargetError


class PermissionGenerator(FilesGenerator):
    def generate(self, target, path_info):
        # the csv file contains lines that match the following layout:
        #    directory,file_name,uid,gid,mode
        dir_path, file_name, uid, gid, mode = path_info

        # build a string out of the path that is suitable for use in id tags
        # example:	/etc/resolv.conf --> _etc_resolv_conf
        # path_id maps to FILEID in the template
        if file_name == '[NULL]':
            path_id = re.sub('[-\./]', '_', dir_path)
        elif re.match(r'\^.*\$', file_name, 0):
            path_id = re.sub('[-\./]', '_', dir_path) + '_' + re.sub('[-\\\./^$*(){}|]',
                                                                     '_', file_name)
            # cleaning trailing end multiple underscores, make sure id is lowercase
            path_id = re.sub(r'_+','_', path_id)
            path_id = re.sub(r'_$','', path_id)
            path_id = path_id.lower()
        else:
            path_id = re.sub('[-\./]', '_', dir_path) + '_' + re.sub('[-\./]',
                                                                     '_', file_name)
            path_id = path_id.lower()

        # build a string that contains the full path to the file
        # full_path maps to FILEPATH in the template
        if file_name == '[NULL]':
            full_path = dir_path
        else:
            full_path = dir_path + '/' + file_name

        if target == "bash":
            if not re.match(r'\^.*\$', file_name, 0):
                self.file_from_template(
                    "./template_BASH_permissions",
                    {
                        "%FILEPATH%":      full_path,
                        "%FILEMODE%":      mode,
                    },
                    "./bash/file_permissions{0}.sh", path_id
                )
            else:
                file_name = re.sub('^\^', '', file_name)
                self.file_from_template(
                    "./template_BASH_regex_permissions",
                    {
                        "%FILEPATH%":      dir_path,
                        "%FILENAME%":      file_name,
                        "%FILEMODE%":      mode,
                    },
                    "./bash/file_permissions{0}.sh", path_id
                )

        elif target == "ansible" and not re.match(r'\^.*\$', file_name, 0):
            self.file_from_template(
                "./template_ANSIBLE_permissions",
                {
                    "%FILEPATH%":      full_path,
                    "%FILEMODE%":      mode,
                },
                "./ansible/file_permissions{0}.yml", path_id
            )

        elif target == "oval":
            # build the state that describes our mode
            # mode_str maps to STATEMODE in the template
            fields = ['oexec', 'owrite', 'oread', 'gexec', 'gwrite', 'gread',
                      'uexec', 'uwrite', 'uread', 'sticky', 'sgid', 'suid']
            mode_int = int(mode, 8)
            mode_str = "  </unix:file_state>"
            for field in fields:
                if mode_int & 0x01 == 1:
                    mode_str = "	<unix:" + field + " datatype=\"boolean\">true</unix:" + field + ">\n" + mode_str
                else:
                    mode_str = "	<unix:" + field + " datatype=\"boolean\">false</unix:" + field + ">\n" + mode_str
                mode_int = mode_int >> 1

            # support pattern matching, requiring that the filename starts with '^'
            # and fichishes with '$'
            if file_name == '[NULL]':
                unix_filename = "<unix:filename xsi:nil=\"true\" />"
            elif re.match( r'\^.*\$', file_name, 0):
                unix_filename = "<unix:filename operation=\"pattern match\">" + file_name + "</unix:filename>"
            else:
                unix_filename = "<unix:filename>" + file_name + "</unix:filename>"

            # we are ready to create the check
            # open the template and perform the conversions
            self.file_from_template(
                "./template_OVAL_permissions",
                {
                    "%FILEID%":        path_id,
                    "%FILEPATH%":      full_path,
                    "%FILEDIR%":       dir_path,
                    "%FILEUID%":       uid,
                    "%FILEGID%":       gid,
                    "%FILEMODE%":      mode,
                    "%STATEMODE%":     mode_str,
                    "%UNIX_FILENAME%": unix_filename
                },
                "./oval/file_permissions{0}.xml", path_id
            )

        else:
            raise UnknownTargetError(target)

    def csv_format(self):
        return("CSV should contains lines of the format: "
               "directory path,file name,owner uid (numeric),group "
               "owner gid (numeric),mode")



#!/usr/bin/env python2
import xml.etree.cElementTree as ET

import logging

NAMESPACES = {
    'xccdf': "http://checklists.nist.gov/xccdf/1.2",
    'ds': "http://scap.nist.gov/schema/scap/source/1.2",
    'xlink': "http://www.w3.org/1999/xlink",
}


logging.getLogger(__name__).addHandler(logging.NullHandler())


def infer_benchmark_id_from_component_ref_id(datastream, ref_id):
    root = ET.parse(datastream).getroot()
    component_ref_node = root.find("*//ds:component-ref[@id='{0}']"
                                   .format(ref_id), NAMESPACES)
    if component_ref_node is None:
        msg = (
            'Component reference of Ref-Id {} not found within datastream'
            .format(ref_id))
        raise RuntimeError(msg)

    comp_id = component_ref_node.get('{%s}href' % NAMESPACES['xlink'])
    comp_id = comp_id.lstrip('#')

    query = ".//ds:component[@id='{}']/xccdf:Benchmark".format(comp_id)
    benchmark_node = root.find(query, NAMESPACES)
    if benchmark_node is None:
        msg = (
            'Benchmark not found within component of Id {}'
            .format(comp_id)
        )
        raise RuntimeError(msg)

    return benchmark_node.get('id')


def get_all_profiles_in_benchmark(datastream, benchmark_id, logging=None):
    root = ET.parse(datastream).getroot()
    benchmark_node = root.find(
        "*//xccdf:Benchmark[@id='{0}']".format(benchmark_id), NAMESPACES)
    if benchmark_node is None:
        if logging is not None:
            logging.error(
                "Benchmark ID '{}' not found within DataStream"
                .format(benchmark_id))
        return []

    all_profiles = benchmark_node.findall('xccdf:Profile', NAMESPACES)
    return all_profiles



#!/usr/bin/env python2

from __future__ import print_function

import fnmatch
import sys
import os

try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    import cElementTree as ElementTree

# Put shared python modules in path
sys.path.insert(0, os.path.join(
        os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
        "modules"))
import idtranslate_module as idtranslate

# This script requires two arguments: an OVAL file and a CPE dictionary file.
# It is designed to extract any inventory definitions and the tests, states,
# objects and variables it references and then write them into a standalone
# OVAL CPE file, along with a synchronized CPE dictionary file.

oval_ns = "http://oval.mitre.org/XMLSchema/oval-definitions-5"
xccdf_ns = "http://checklists.nist.gov/xccdf/1.1"
cpe_ns = "http://cpe.mitre.org/dictionary/2.0"


def parse_xml_file(xmlfile):
    with open(xmlfile, 'r') as xml_file:
        filestring = xml_file.read()
        tree = ElementTree.fromstring(filestring)
    return tree


def extract_subelement(objects, sub_elem_type):
    for obj in objects:
        for subelement in obj.getiterator():
            if subelement.get(sub_elem_type):
                sub_element = subelement.get(sub_elem_type)
                return sub_element


def extract_env_obj(objects, local_var):
    for obj in objects:
        env_id = extract_subelement(local_var, 'object_ref')
        if env_id == obj.get('id'):
            return obj


def extract_referred_nodes(tree_with_refs, tree_with_ids, attrname):
    reflist = []
    elementlist = []
    iter = tree_with_refs.getiterator()
    for element in iter:
        value = element.get(attrname)
        if value is not None:
            reflist.append(value)

    iter = tree_with_ids.getiterator()
    for element in iter:
        if element.get("id") in reflist:
            elementlist.append(element)
    return elementlist


def collect_nodes(tree, reflist):
    elementlist = []
    iter = tree.getiterator()
    for element in iter:
        if element.get("id") in reflist:
            elementlist.append(element)
    return elementlist


def main():
    if len(sys.argv) < 2:
        print("Provide an OVAL file that contains inventory definitions.")
        print("This script extracts these definitions and writes them "
              "to STDOUT.")
        sys.exit(1)

    product = sys.argv[1]
    idname = sys.argv[2]
    cpeoutdir = sys.argv[3]
    ovalfile = sys.argv[4]
    cpedictfile = sys.argv[5]

    # parse oval file
    ovaltree = parse_xml_file(ovalfile)

    # extract inventory definitions
    # making (dubious) assumption that all inventory defs are CPE
    defs = ovaltree.find("./{%s}definitions" % oval_ns)
    inventory_defs = []
    for el in defs.findall(".//{%s}definition" % oval_ns):
        if el.get("class") != "inventory":
            continue
        inventory_defs.append(el)

    # Keep the list of 'id' attributes from untranslated inventory def elements
    inventory_defs_id_attrs = []

    defs.clear()
    [defs.append(inventory_def) for inventory_def in inventory_defs]
    # Fill in that list
    inventory_defs_id_attrs = \
        [inventory_def.get("id") for inventory_def in inventory_defs]

    tests = ovaltree.find("./{%s}tests" % oval_ns)
    cpe_tests = extract_referred_nodes(defs, tests, "test_ref")
    tests.clear()
    [tests.append(cpe_test) for cpe_test in cpe_tests]

    states = ovaltree.find("./{%s}states" % oval_ns)
    cpe_states = extract_referred_nodes(tests, states, "state_ref")
    states.clear()
    [states.append(cpe_state) for cpe_state in cpe_states]

    objects = ovaltree.find("./{%s}objects" % oval_ns)
    cpe_objects = extract_referred_nodes(tests, objects, "object_ref")
    env_objects = extract_referred_nodes(objects, objects, "id")
    objects.clear()
    [objects.append(cpe_object) for cpe_object in cpe_objects]

    # if any subelements in an object contain var_ref, return it here
    local_var_ref = extract_subelement(objects, 'var_ref')

    variables = ovaltree.find("./{%s}variables" % oval_ns)
    if variables is not None:
        cpe_variables = extract_referred_nodes(tests, variables, "var_ref")
        local_variables = extract_referred_nodes(variables, variables, "id")
        if cpe_variables:
            variables.clear()
            [variables.append(cpe_variable) for cpe_variable in cpe_variables]
        elif local_var_ref:
            for local_var in local_variables:
                if local_var.get('id') == local_var_ref:
                    variables.clear()
                    variables.append(local_var)
                    env_obj = extract_env_obj(env_objects, local_var)
                    objects.append(env_obj)
        else:
            ovaltree.remove(variables)

    # turn IDs into meaningless numbers
    translator = idtranslate.IDTranslator(idname)
    ovaltree = translator.translate(ovaltree)

    newovalfile = idname + "-" + product + "-" + os.path.basename(ovalfile)
    newovalfile = newovalfile.replace("oval-unlinked", "cpe-oval")
    ElementTree.ElementTree(ovaltree).write(cpeoutdir + "/" + newovalfile)

    # replace and sync IDs, href filenames in input cpe dictionary file
    cpedicttree = parse_xml_file(cpedictfile)
    newcpedictfile = idname + "-" + os.path.basename(cpedictfile)
    for check in cpedicttree.findall(".//{%s}check" % cpe_ns):
        checkhref = check.get("href")
        # If CPE OVAL references another OVAL file
        if checkhref == 'filename':
            # Sanity check -- Verify the referenced OVAL is truly defined
            # somewhere in the (sub)directory tree below CWD. In correct
            # scenario is should be located:
            # * either in input/oval/*.xml
            # * or copied by former run of "combine-ovals.py" script from
            #   shared/ directory into build/ subdirectory
            refovalfilename = check.text
            refovalfilefound = False
            for dirpath, dirnames, filenames in os.walk(os.curdir, topdown=True):
                # Case when referenced OVAL file exists
                for location in fnmatch.filter(filenames, refovalfilename + '.xml'):
                    refovalfilefound = True
                    break                     # break from the inner for loop

                if refovalfilefound:
                    break                     # break from the outer for loop

            shared_dir = \
                os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
            if shared_dir is not None:
                for dirpath, dirnames, filenames in os.walk(shared_dir, topdown=True):
                    # Case when referenced OVAL file exists
                    for location in fnmatch.filter(filenames, refovalfilename + '.xml'):
                        refovalfilefound = True
                        break                     # break from the inner for loop

                    if refovalfilefound:
                        break                     # break from the outer for loop

            # Referenced OVAL doesn't exist in the subdirtree below CWD:
            # * there's either typo in the refenced OVAL filename, or
            # * is has been forgotten to be placed into input/oval, or
            # * the <platform> tag of particular shared/ OVAL wasn't modified
            #   to include the necessary referenced file.
            # Therefore display an error and exit with failure in such cases
            if not refovalfilefound:
                error_msg = "\n\tError: Can't locate \"%s\" OVAL file in the \
                \n\tlist of OVAL checks for this product! Exiting..\n" % refovalfilename
                sys.stderr.write(error_msg)
                # sys.exit(1)
        check.set("href", os.path.basename(newovalfile))

        # Sanity check to verify if inventory check OVAL id is present in the
        # list of known "id" attributes of inventory definitions. If not it
        # means provided ovalfile (sys.argv[1]) doesn't contain this OVAL
        # definition (it wasn't included due to <platform> tag restrictions)
        # Therefore display an error and exit with failure, since otherwise
        # we might end up creating invalid $(ID)-$(PROD)-cpe-oval.xml file
        if check.text not in inventory_defs_id_attrs:
            error_msg = "\n\tError: Can't locate \"%s\" definition in \"%s\". \
            \n\tEnsure <platform> element is configured properly for \"%s\".  \
            \n\tExiting..\n" % (check.text, ovalfile, check.text)
            sys.stderr.write(error_msg)
            # sys.exit(1)

        # Referenced OVAL checks passed both of the above sanity tests
        check.text = translator.generate_id("{" + oval_ns + "}definition", check.text)

    ElementTree.ElementTree(cpedicttree).write(cpeoutdir + '/' + newcpedictfile)

    sys.exit(0)

if __name__ == "__main__":
    main()



import re

# SSG Makefile to official product name mapping
CHROMIUM = 'Google Chromium Browser'
FEDORA = 'Fedora'
FIREFOX = 'Mozilla Firefox'
JRE = 'Java Runtime Environment'
RHEL = 'Red Hat Enterprise Linux'
DEBIAN = 'Debian'
UBUNTU = 'Ubuntu'
EAP = 'JBoss Enterprise Application Platform'
FUSE = 'JBoss Fuse'
OPENSUSE = 'openSUSE'
SUSE = 'SUSE Linux Enterprise'
WRLINUX = 'Wind River Linux'
OL = 'Oracle Linux'

multi_product_list = ["rhel", "fedora", "rhel-osp", "debian", "ubuntu",
                      "wrlinux", "opensuse", "sle", "ol"]

PRODUCT_NAME_PARSER = re.compile("([a-zA-Z\-]+)([0-9]+)")


def parse_product_name(product):
    product_version = None
    match = PRODUCT_NAME_PARSER.match(product)

    if match is not None:
        if isinstance(match.group(1), str) or \
                isinstance(match.group(1), unicode):
            product = match.group(1)
        if match.group(2).isdigit():
            product_version = match.group(2)

    return product, product_version


def map_product(version):
    """Maps SSG Makefile internal product name to official product name"""

    if version.startswith("multi_platform_"):
        trimmed_version = version[len("multi_platform_"):]
        if trimmed_version not in multi_product_list:
            raise RuntimeError(
                "%s is an invalid product version. If it's multi_platform the "
                "suffix has to be from (%s)."
                % (version, ", ".join(multi_product_list))
            )
        return map_product(trimmed_version)

    if version.startswith("chromium"):
        return CHROMIUM
    if version.startswith("fedora"):
        return FEDORA
    if version.startswith("firefox"):
        return FIREFOX
    if version.startswith("jre"):
        return JRE
    if version.startswith("rhel"):
        return RHEL
    if version.startswith("debian"):
        return DEBIAN
    if version.startswith("ubuntu"):
        return UBUNTU
    if version.startswith("eap"):
        return EAP
    if version.startswith("fuse"):
        return FUSE
    if version.startswith("opensuse"):
        return OPENSUSE
    if version.startswith("sle"):
        return SUSE
    if version.startswith("wrlinux"):
        return WRLINUX
    if version.startswith("ol"):
        return OL


    raise RuntimeError("Can't map version '%s' to any known product!"
                       % (version))



#!/usr/bin/env python2
from __future__ import print_function
import logging
import os.path
import re
import subprocess
import collections
import xml.etree.ElementTree
import sys
import json

from ssg_test_suite.log import LogHelper

logging.getLogger(__name__).addHandler(logging.NullHandler())

_CONTEXT_RETURN_CODES = {'pass': 0,
                         'fail': 2,
                         'error': 1,
                         'notapplicable': 0,
                         'fixed': 0}

_ANSIBLE_TEMPLATE = 'urn:xccdf:fix:script:ansible'
_BASH_TEMPLATE = 'urn:xccdf:fix:script:sh'
_XCCDF_NS = 'http://checklists.nist.gov/xccdf/1.2'


def analysis_to_serializable(analysis):
    result = dict(analysis)
    for key, value in analysis.items():
        if type(value) == set:
            result[key] = tuple(value)
    return result


def save_analysis_to_json(analysis, output_fname):
    analysis2 = analysis_to_serializable(analysis)
    with open(output_fname, "w") as f:
        json.dump(analysis2, f)


def triage_xml_results(fname):
    tree = xml.etree.ElementTree.parse(fname)
    all_xml_results = tree.findall(".//{%s}rule-result" % _XCCDF_NS)

    triaged = collections.defaultdict(set)
    for result in list(all_xml_results):
        idref = result.get("idref")
        status = result.find("{%s}result" % _XCCDF_NS).text
        triaged[status].add(idref)

    return triaged


def run_cmd(command, verbose_path):
    command_string = ' '.join(command)
    logging.debug('Running {}'.format(command_string))
    returncode = 0
    output = ""
    try:
        with open(verbose_path, 'w') as verbose_file:
            output = subprocess.check_output(command,
                                             stderr=verbose_file)
    except subprocess.CalledProcessError as e:
        returncode = e.returncode
        output = e.output
    return returncode, output


def run_cmd_remote(command_string, domain_ip, verbose_path):
    machine = 'root@{0}'.format(domain_ip)
    remote_cmd = ['ssh', machine, command_string]
    logging.debug('Running {}'.format(command_string))
    returncode = 0
    output = ""
    try:
        with open(verbose_path, 'w') as verbose_file:
            output = subprocess.check_output(remote_cmd,
                                             stderr=verbose_file)
    except subprocess.CalledProcessError as e:
        returncode = e.returncode
        output = e.output
    return returncode, output


def send_files_remote(verbose_path, remote_dir, domain_ip, *files):
    """Upload files to VM."""
    # files is a list of absolute paths on the host
    success = True
    destination = 'root@{0}:{1}'.format(domain_ip, remote_dir)
    files_string = ' '.join(files)

    logging.debug('Uploading files {0} to {1}'.format(files_string,
                                                      destination))
    command = ['scp'] + list(files) + [destination]
    if run_cmd(command, verbose_path)[0] != 0:
        logging.error('Failed to upload files {0}'.format(files_string))
        success = False
    return success


def get_file_remote(verbose_path, local_dir, domain_ip, remote_path):
    """Download a file from VM."""
    # remote_path is an absolute path of a file on remote machine
    success = True
    source = 'root@{0}:{1}'.format(domain_ip, remote_path)
    logging.debug('Downloading file {0} to {1}'
                  .format(source, local_dir))
    command = ['scp', source, local_dir]
    if run_cmd(command, verbose_path)[0] != 0:
        logging.error('Failed to download file {0}'.format(remote_path))
        success = False
    return success


def find_result_id_in_output(output):
    match = re.search('result id.*$', output, re.IGNORECASE | re.MULTILINE)
    if match is None:
        return None
    # Return the right most word of the match which is the result id.
    return match.group(0).split()[-1]


def ansible_playbook_set_hosts(playbook):
    """Updates ansible playbok to apply to all hosts."""
    with open(playbook, 'r') as f:
        lines = f.readlines()
    lines.insert(1, ' - hosts: all\n')
    with open(playbook, 'w') as f:
        for line in lines:
            f.write(line)


def get_result_id_from_arf(arf_path, verbose_path):
    command = ['oscap', 'info', arf_path]
    command_string = ' '.join(command)
    returncode, output = run_cmd(command, verbose_path)
    if returncode != 0:
        raise RuntimeError('{0} returned {1} exit code'.
                           format(command_string, returncode))
    res_id = find_result_id_in_output(output)
    if res_id is None:
        raise RuntimeError('Failed to find result ID in {0}'
                           .format(arf_path))
    return res_id


def generate_fixes_remotely(formatting, verbose_path):
    command_base = ['oscap', 'xccdf', 'generate', 'fix']
    command_options = [
        '--benchmark-id', formatting['benchmark_id'],
        '--profile', formatting['profile'],
        '--template', formatting['output_template'],
        '--output', '/{output_file}'.format(** formatting),
    ]
    command_operands = ['/{arf_file}'.format(** formatting)]
    if 'result_id' in formatting:
        command_options.extend(['--result-id', formatting['result_id']])

    command_string = ' '.join(command_base + command_options + command_operands)
    rc, stdout = run_cmd_remote(command_string,
                                formatting['domain_ip'], verbose_path)
    if rc != 0:
        msg = ('Command {0} ended with return code {1} (expected 0).'
               .format(command_string, rc))
        raise RuntimeError(msg)


def run_stage_remediation_ansible(run_type, formatting, verbose_path):
    """
       Returns False on error, or True in case of successful bash scripts
       run."""
    formatting['output_template'] = _ANSIBLE_TEMPLATE
    send_arf_to_remote_machine_and_generate_remediations_there(
        run_type, formatting, verbose_path)
    if not get_file_remote(verbose_path, LogHelper.LOG_DIR,
                           formatting['domain_ip'],
                           '/' + formatting['output_file']):
        return False
    ansible_playbook_set_hosts(formatting['playbook'])
    command = (
        'ansible-playbook',  '-i', '{0},'.format(formatting['domain_ip']),
        '-u' 'root', formatting['playbook'])
    command_string = ' '.join(command)
    returncode, output = run_cmd(command, verbose_path)
    # Appends output of ansible-playbook to the verbose_path file.
    with open(verbose_path, 'a') as f:
        f.write('Stdout of "{}":'.format(command_string))
        f.write(output)
    if returncode != 0:
        msg = (
            'Ansible playbook remediation run has '
            'exited with return code {} instead of expected 0'
            .format(returncode))
        LogHelper.preload_log(logging.ERROR, msg, 'fail')
        return False
    return True


def run_stage_remediation_bash(run_type, formatting, verbose_path):
    """
       Returns False on error, or True in case of successful Ansible playbook
       run."""
    formatting['output_template'] = _BASH_TEMPLATE
    send_arf_to_remote_machine_and_generate_remediations_there(
        run_type, formatting, verbose_path)
    if not get_file_remote(verbose_path, LogHelper.LOG_DIR,
                           formatting['domain_ip'],
                           '/' + formatting['output_file']):
        return False

    command_string = '/bin/bash /{output_file}'.format(** formatting)
    returncode, output = run_cmd_remote(
        command_string, formatting['domain_ip'], verbose_path)
    # Appends output of script execution to the verbose_path file.
    with open(verbose_path, 'a') as f:
        f.write('Stdout of "{}":'.format(command_string))
        f.write(output)
    if returncode != 0:
        msg = (
            'Bash script remediation run has exited with return code {} '
            'instead of expected 0'.format(returncode))
        LogHelper.preload_log(logging.ERROR, msg, 'fail')
        return False
    return True


def send_arf_to_remote_machine_and_generate_remediations_there(
        run_type, formatting, verbose_path):
    if run_type == 'rule':
        try:
            res_id = get_result_id_from_arf(formatting['arf'], verbose_path)
        except Exception as exc:
            logging.error(str(exc))
            return False
        formatting['result_id'] = res_id

    if not send_files_remote(
            verbose_path, '/', formatting['domain_ip'], formatting['arf']):
        return False

    try:
        generate_fixes_remotely(formatting, verbose_path)
    except Exception as exc:
        logging.error(str(exc))
        return False


class GenericRunner(object):
    def __init__(self, domain_ip, profile, datastream, benchmark_id):
        self.domain_ip = domain_ip
        self.profile = profile
        self.datastream = datastream
        self.benchmark_id = benchmark_id

        self.arf_file = ''
        self.arf_path = ''
        self.verbose_path = ''
        self.report_path = ''
        self.results_path = ''
        self.stage = 'undefined'

        self.clean_files = False
        self._filenames_to_clean_afterwards = set()

        self.command_base = []
        self.command_options = []
        self.command_operands = []

    def _make_arf_path(self):
        self.arf_file = self._get_arf_file()
        self.arf_path = os.path.join(LogHelper.LOG_DIR, self.arf_file)

    def _get_arf_file(self):
        raise NotImplementedError()

    def _make_verbose_path(self):
        verbose_file = self._get_verbose_file()
        verbose_path = os.path.join(LogHelper.LOG_DIR, verbose_file)
        self.verbose_path = LogHelper.find_name(verbose_path, '.verbose.log')

    def _get_verbose_file(self):
        raise NotImplementedError()

    def _make_report_path(self):
        report_file = self._get_report_file()
        report_path = os.path.join(LogHelper.LOG_DIR, report_file)
        self.report_path = LogHelper.find_name(report_path, '.html')

    def _get_report_file(self):
        raise NotImplementedError()

    def _make_results_path(self):
        results_file = self._get_results_file()
        results_path = os.path.join(LogHelper.LOG_DIR, results_file)
        self.results_path = LogHelper.find_name(results_path, '.xml')

    def _get_results_file(self):
        raise NotImplementedError()

    def _generate_report_file(self):
        self.command_options.extend([
            '--report', self.report_path,
        ])
        self._filenames_to_clean_afterwards.add(self.report_path)

    def prepare_oscap_ssh_arguments(self):
        full_hostname = 'root@{}'.format(self.domain_ip)
        self.command_base.extend(
            ['oscap-ssh', full_hostname, '22', 'xccdf', 'eval'])
        self.command_options.extend([
            '--benchmark-id', self.benchmark_id,
            '--profile', self.profile,
            '--verbose', 'DEVEL',
            '--progress', '--oval-results',
        ])
        self.command_operands.append(self.datastream)

    def run_stage(self, stage):
        self.stage = stage

        self._make_verbose_path()
        self._make_report_path()
        self._make_arf_path()
        self._make_results_path()

        self.command_base = []
        self.command_options = []
        self.command_operands = []

        result = None
        if stage == 'initial':
            result = self.initial()
        elif stage == 'remediation':
            result = self.remediation()
        elif stage == 'final':
            result = self.final()
        else:
            raise RuntimeError('Unknown stage: {}.'.format(stage))

        if self.clean_files:
            for fname in self._filenames_to_clean_afterwards:
                os.remove(fname)

        if result:
            LogHelper.log_preloaded('pass')
        else:
            LogHelper.log_preloaded('fail')
        return result

    @property
    def get_command(self):
        return self.command_base + self.command_options + self.command_operands

    def make_oscap_call(self):
        raise NotImplementedError()

    def initial(self):
        self.command_options += ['--results', self.results_path]
        result = self.make_oscap_call()
        return result

    def remediation(self):
        raise NotImplementedError()

    def final(self):
        self.command_options += ['--results', self.results_path]
        result = self.make_oscap_call()
        return result

    def analyze(self, stage):
        triaged_results = triage_xml_results(self.results_path)
        triaged_results["stage"] = stage
        triaged_results["runner"] = self.__class__.__name__
        return triaged_results

    def _get_formatting_dict_for_remediation(self):
        formatting = {
            'domain_ip': self.domain_ip,
            'profile': self.profile,
            'datastream': self.datastream,
            'benchmark_id': self.benchmark_id
        }
        formatting['arf'] = self.arf_path
        formatting['arf_file'] = self.arf_file
        return formatting


class ProfileRunner(GenericRunner):
    def _get_arf_file(self):
        return '{0}-initial-arf.xml'.format(self.profile)

    def _get_verbose_file(self):
        return '{0}-{1}'.format(self.profile, self.stage)

    def _get_report_file(self):
        return '{0}-{1}'.format(self.profile, self.stage)

    def _get_results_file(self):
        return '{0}-{1}-results'.format(self.profile, self.stage)

    def make_oscap_call(self):
        self.prepare_oscap_ssh_arguments()
        self._generate_report_file()
        returncode = run_cmd(self.get_command, self.verbose_path)[0]
        if returncode not in [0, 2]:
            logging.error(('Profile run should end with return code 0 or 2 '
                           'not "{0}" as it did!').format(returncode))
            return False
        return True


class RuleRunner(GenericRunner):
    def __init__(
            self, domain_ip, profile, datastream, benchmark_id,
            rule_id, context, script_name, dont_clean):
        super(RuleRunner, self).__init__(
            domain_ip, profile, datastream, benchmark_id,
        )

        self.rule_id = rule_id
        self.context = context
        self.script_name = script_name
        self.clean_files = not dont_clean

        self._oscap_output = ''

    def _get_arf_file(self):
        return '{0}-initial-arf.xml'.format(self.rule_id)

    def _get_verbose_file(self):
        return '{0}-{1}-{2}'.format(self.rule_id, self.script_name, self.stage)

    def _get_report_file(self):
        return '{0}-{1}-{2}'.format(self.rule_id, self.script_name, self.stage)

    def _get_results_file(self):
        return '{0}-{1}-{2}-results-{3}'.format(
            self.rule_id, self.script_name, self.profile, self.stage)

    def make_oscap_call(self):
        self.prepare_oscap_ssh_arguments()
        self._generate_report_file()
        self.command_options.extend(
            ['--rule', self.rule_id])

        returncode, self._oscap_output = run_cmd(self.get_command, self.verbose_path)

        expected_return_code = _CONTEXT_RETURN_CODES[self.context]

        if returncode != expected_return_code:
            msg = (
                'Scan has exited with return code {0}, '
                'instead of expected {1} during stage {2}'
                .format(returncode, expected_return_code, self.stage)
            )
            LogHelper.preload_log(logging.ERROR, msg, 'fail')
            return False
        return True

    def final(self):
        success = super(RuleRunner, self).final()
        success = success and self._analyze_output_of_oscap_call()

        return success

    def _analyze_output_of_oscap_call(self):
        local_success = True
        # check expected result
        actual_results = re.findall('{0}:(.*)$'.format(self.rule_id),
                                    self._oscap_output,
                                    re.MULTILINE)
        if actual_results:
            if self.context not in actual_results:
                LogHelper.preload_log(logging.ERROR,
                                      ('Rule result should have been '
                                       '"{0}", but is "{1}"!'
                                       ).format(self.context,
                                                ', '.join(actual_results)),
                                      'fail')
                local_success = False
        else:
            msg = (
                'Rule {0} has not been evaluated! Wrong profile selected?'
                .format(self.rule_id))
            LogHelper.preload_log(logging.ERROR, msg, 'fail')
            local_success = False
        return local_success

    def _get_formatting_dict_for_remediation(self):
        fmt = super(RuleRunner, self)._get_formatting_dict_for_remediation()
        fmt['rule_id'] = self.rule_id

        return fmt


class OscapProfileRunner(ProfileRunner):
    def remediation(self):
        self.command_options += ['--remediate']
        return self.make_oscap_call()


class AnsibleProfileRunner(ProfileRunner):
    def initial(self):
        self.command_options += ['--results-arf', self.arf_path]
        return super(AnsibleProfileRunner, self).initial()

    def remediation(self):
        formatting = self._get_formatting_dict_for_remediation()
        formatting['output_file'] = '{0}.yml'.format(self.profile)
        formatting['playbook'] = os.path.join(LogHelper.LOG_DIR,
                                              formatting['output_file'])

        return run_stage_remediation_ansible('profile', formatting, self.verbose_path)


class BashProfileRunner(ProfileRunner):
    def initial(self):
        self.command_options += ['--results-arf', self.arf_path]
        return super(BashProfileRunner, self).initial()

    def remediation(self):
        formatting = self._get_formatting_dict_for_remediation()
        formatting['output_file'] = '{0}.sh'.format(self.profile)

        return run_stage_remediation_bash('profile', formatting, self.verbose_path)


class OscapRuleRunner(RuleRunner):
    def remediation(self):
        self.command_options += ['--remediate']
        return self.make_oscap_call()


class BashRuleRunner(RuleRunner):
    def initial(self):
        self.command_options += ['--results-arf', self.arf_path]
        return super(BashRuleRunner, self).initial()

    def remediation(self):
        formatting = self._get_formatting_dict_for_remediation()
        formatting['output_file'] = '{0}.sh'.format(self.rule_id)

        success = run_stage_remediation_bash('rule', formatting, self.verbose_path)
        return success


class AnsibleRuleRunner(RuleRunner):
    def initial(self):
        self.command_options += ['--results-arf', self.arf_path]
        return super(AnsibleRuleRunner, self).initial()

    def remediation(self):
        formatting = self._get_formatting_dict_for_remediation()
        formatting['output_file'] = '{0}.yml'.format(self.rule_id)
        formatting['playbook'] = os.path.join(LogHelper.LOG_DIR,
                                              formatting['output_file'])

        success = run_stage_remediation_ansible('rule', formatting, self.verbose_path)
        return success


def run_profile(domain_ip,
                profile,
                stage,
                datastream,
                benchmark_id,
                runner):
    """Run `oscap-ssh` command with provided parameters to check given profile.
    Log output into LogHelper.LOG_DIR.

    Return True if command ends with exit codes 0 or 2 for bash remediations or
    with 0 for Ansible remediations, otherwise return False.
    """
    runner_cls = REMEDIATION_PROFILE_RUNNERS[runner]
    runner = runner_cls(
        domain_ip, profile, datastream, benchmark_id)
    success = runner.run_stage(stage)
    return success


def run_rule(domain_ip,
             profile,
             stage,
             datastream,
             benchmark_id,
             rule_id,
             context,
             script_name,
             runner,
             dont_clean=False):
    """Run `oscap-ssh` command with provided parameters to check given rule,
    utilizing --rule option. Log output to LogHelper.LOG_DIR directory.

    Return True if result is as expected by context parameter. Check both
    exit code and output message.
    """
    runner_cls = REMEDIATION_RULE_RUNNERS[runner]
    runner = runner_cls(
        domain_ip, profile, datastream, benchmark_id,
        rule_id, context, script_name, dont_clean)
    success = runner.run_stage(stage)
    return success


REMEDIATION_PROFILE_RUNNERS = {
    'oscap': OscapProfileRunner,
    'bash': BashProfileRunner,
    'ansible': AnsibleProfileRunner,
}


REMEDIATION_RULE_RUNNERS = {
    'oscap': OscapRuleRunner,
    'bash': BashRuleRunner,
    'ansible': AnsibleRuleRunner,
}


REMEDIATION_RUNNER_TO_REMEDIATION_MEANS = {
    'oscap': 'bash',
    'bash': 'bash',
    'ansible': 'ansible',
}
